/***************************************************************************
                          arcitem.cpp  -  description
                             -------------------
    begin                : Sam Aug 17 2002
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <KConfigGroup>
#include <KLocalizedString>
#include "arcitem.h"
#include "kplscene.h"

ArcItem::ArcItem(double x, double y, unsigned color,
                 QGraphicsItem* parent, double w, double h, bool visible,
                 double start, double span, double relSize, double angle) :
 DecoItem(color, x, y, relSize), QGraphicsPathItem(parent), m_width(w),
 m_height(h), m_dir(angle), m_start(start), m_span(span)
{
  if (!visible)
    hide();
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

ArcItem::ArcItem(const ArcItem& a) :
 DecoItem(a), QGraphicsPathItem(), m_width(a.m_width), m_height(a.m_height),
 m_dir(a.m_dir), m_start(a.m_start), m_span(a.m_span)
{
  setZValue(a.zValue());
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
  setSelected(a.isSelected());
  setVisible(a.isVisible());
}

ArcItem::ArcItem(KConfigGroup* plo, unsigned color, QGraphicsItem* parent) :
 DecoItem(plo, color), QGraphicsPathItem(parent)
{
  if (!plo->readEntry("active", true))
    hide();
  m_width = plo->readEntry("width", 3.0);
  m_height = plo->readEntry("height", 2.0);
  m_dir = plo->readEntry("dir", 0.0);
  m_start = plo->readEntry("a", 0.0);
  m_span = plo->readEntry("alen", 90.0);
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

ArcItem::~ArcItem()
{
}

QPixmap ArcItem::pixmap() const
{
  return KplItem::pixmap(pen(), 1);
}

void ArcItem::setPenWidth()
{
  QPen p = pen();
  double w = penWidth(this);
  if (p.widthF() != w) {
    prepareGeometryChange();
    setPen(changePenWidth(p, w));
  }
}

void ArcItem::setProperties(const QTransform& m)
{
  m_matrix = m;
  setProperties();
}

QString ArcItem::settings() const
{
  return QString("x = %1 cm, y = %2 cm, ").arg(m_x0).arg(m_y0) + i18n("size") +
    QString(" %1 cm x %2 cm").arg(m_width).arg(m_height);
}

void ArcItem::update()
{
  prepareGeometryChange();
  setProperties();
  QGraphicsItem::update();
}

void ArcItem::writePlo(KConfigGroup* plo) const
{
  plo->writeEntry("Type", "ARCITEM");
  DecoItem::writePlo(plo, isVisible());
  plo->writeEntry("width", m_width);
  plo->writeEntry("height", m_height);
  plo->writeEntry("dir", m_dir);
  plo->writeEntry("a", m_start);
  plo->writeEntry("alen", m_span);
}

void ArcItem::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
  QGraphicsPathItem::mouseReleaseEvent(event);
  ((KplScene*)scene())->update(this);
}

void ArcItem::setProperties()
{
  setPos(m_matrix.map(QPointF(m_x0, m_y0)));
#if QT_VERSION < 0x040600
  setTransform(QTransform().rotate(-m_dir));
#else
  setRotation(-m_dir);
#endif
  setPen(createPen(this, m_color));
  double w = m_matrix.m11() * m_width;
  double h = m_matrix.m22() * m_height;
  QRectF r(-0.5 * w, -0.5 * h, w, h);
  QPainterPath path;
  path.arcMoveTo(r, -m_start);
  path.arcTo(r, -m_start, -m_span);
  setPath(path);
}
