/***************************************************************************
                          arraydlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ARRAYDLG_H
#define ARRAYDLG_H

#include <QString>
#include <KDialog>

class QCheckBox;
class QLabel;
class QToolButton;
class KColorButton;
class KUrl;
class ArrayItem;
class BrushButton;
class KplDoc;
class DoubleEdit;
class DoubleSpinBox;
class SpinBox;

/**
  * Array settings dialog class. Provides a dialog for the settings
  * of an array plot item.
  * @author Werner Stille
  */
class ArrayDlg : public KDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param ai pointer to array object.
   * @param pos dialog position.
   */
  ArrayDlg(QWidget* parent, KplDoc* model, ArrayItem* ai, QPoint pos);
  /** Destructor. */
  ~ArrayDlg();

protected:
  /**
   * Loads file.
   * @param url URL of file.
   * @param idec 0 for decimal point, 1 for comma.
   * @return true when when succeeded.
   */
  bool load(const KUrl& url, int idec);
  /**
   * Reads edited data into array object.
   * @param fromBuffer read editor buffer instead of file when true.
   * @return true when when succeeded.
   */
  bool readData(bool fromBuffer = false);
  /**
   * Updates ranges of spin boxes.
   * @param ad pointer to array object.
   */
  void setSpinBoxes(ArrayItem* ad);
  QString buf;
  KplDoc* m;
  ArrayItem *ai, *ait;
  DoubleEdit *efx, *efy;
  DoubleSpinBox *ex0, *ey0;
  QLabel* fileName;
  SpinBox *ix, *iy, *ie, *iStart, *n, *symb, *eRelSize;
  QCheckBox *err, *internal;
  BrushButton *fill;
  KColorButton* colData;
  QPushButton *edit;
  QToolButton *bReload;

protected slots:
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /** Reimplemented from KDialog. */
  virtual void slotButtonClicked(int button);
  /** Edits internal data. */
  void slotEdit();
  /** Enables buttons. */
  void slotEdited();
  /** Applies edited array data. */
  void slotEditApply();
  /** Displays the dialog for selection of a data file. */
  void slotFile();
  /** Toggles internal/external data. */
  void slotInternal(bool state);
  /** Adjusts the Start index spinbox when number of points is changed. */
  void slotNChanged(int n);
  /** Reloads file. */
  void slotReload();
  /** Adjusts the Number of points spinbox when start index is changed. */
  void slotStartChanged(int);
};

#endif
