/***************************************************************************
                          arrowdlg.cpp  -  description
                             -------------------
    begin                : Sat Apr 14 2001
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QLayout>
#include <QRadioButton>
#include <KButtonGroup>
#include <KColorButton>
#include <KLocalizedString>
#include <KUndoStack>
#include "arrowdlg.h"
#include "arrowitem.h"
#include "kpldoc.h"
#include "spinbox.h"

ArrowDlg::ArrowDlg(QWidget* parent, KplDoc* model, ArrowItem* ai0,
                   QPoint pos) :
 DecoDlg(parent, i18n("Arrow"), "SEC-ARROW", model, ai0, pos), koord(0),
 ai(ai0)
{
  initPolarEntries(ai, grid, &eDir, &eLen);
  grid->addWidget(new QLabel(i18n("Coordinates")), 1, 0);
  KButtonGroup* bg = new KButtonGroup;
  grid->addWidget(bg, 1, 1, 1, 6);
  bg->setFlat(true);
  QHBoxLayout* hbox = new QHBoxLayout(bg);
  hbox->addWidget(new QRadioButton(i18n("Head")));
  hbox->addWidget(new QRadioButton(i18n("Center")));
  hbox->addWidget(new QRadioButton(i18n("End")));
  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::Expanding));
  bg->setSelected(0);
  initEntries(2);
  grid->addWidget(dbl = new QCheckBox(i18n("Double headed")), 4, 0, 1, 7);
  dbl->setChecked(ai->doubleHeaded());
  connect(bg, SIGNAL(changed(int)), SLOT(slotSetCoord(int)));
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eLen, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(dbl, SIGNAL(toggled(bool)), SLOT(slotEdited()));
}

ArrowDlg::~ArrowDlg()
{
}

void ArrowDlg::initPolarEntries(VectorItem* ai, QGridLayout* gr,
                                QDoubleSpinBox** eD, QDoubleSpinBox** eL)
{
  gr->addWidget(new QLabel(i18n("Direction")), 0, 0);
  gr->addWidget(*eD = new QDoubleSpinBox, 0, 1);
  (*eD)->setRange(-360, 360);
  (*eD)->setSuffix("\260");
  (*eD)->setValue(ai->dir());
  gr->addWidget(new QLabel(i18n("Length")), 0, 3);
  gr->addWidget(*eL = new QDoubleSpinBox, 0, 4);
  (*eL)->setMaximum(200);
  (*eL)->setSingleStep(0.1);
  (*eL)->setSuffix(" cm");
  (*eL)->setValue(ai->len());
}

void ArrowDlg::calcCoord(double* x, double* y, double dx, double dy)
{
  switch (koord) {
    case 1:
      *x += 0.5 * dx;
      *y += 0.5 * dy;
      break;
    case 2:
      *x += dx;
      *y += dy;
  }
}

void ArrowDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      ArrowItem* aiNew = new ArrowItem(*ai);
      getValues(aiNew);
      aiNew->setDoubleHeaded(dbl->isChecked());
      double dir = eDir->value();
      aiNew->setDir(dir);
      double len = eLen->value();
      aiNew->setLen(len);
      dir *= VectorItem::deg2rad;
      double x0 = aiNew->x0();
      double y0 = aiNew->y0();
      calcCoord(&x0, &y0, len * cos(dir), len * sin(dir));
      aiNew->setX0(0.01 * qRound(100 * x0));
      aiNew->setY0(0.01 * qRound(100 * y0));
      m->undoStack()->push(new EditCommand(aiNew, ai, m));
      ai = aiNew;
      if (button == Ok)
        accept();
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}

void ArrowDlg::slotSetCoord(int id)
{
  if (id != koord) {
    double dir = VectorItem::deg2rad * eDir->value();
    double len = eLen->value();
    double dx = len * cos(dir);
    double dy = len * sin(dir);
    double x0 = ex0->value();
    double y0 = ey0->value();
    calcCoord(&x0, &y0, dx, dy);
    koord = id;
    calcCoord(&x0, &y0, -dx, -dy);
    ex0->setValue(x0);
    ey0->setValue(y0);
  }
}
