/***************************************************************************
                          brushdlg.h  -  description
                             -------------------
    begin                : Mit Mai 14 2003
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BRUSHDLG_H
#define BRUSHDLG_H

#include <QPushButton>
#include <QRadioButton>
#include <KDialog>

class QPainter;
class KButtonGroup;

/**
  * Brush button. Provides a button which shows the current
  * brush fill pattern and opens a dialog to select a pattern.
  * @author Werner Stille
  */
class BrushButton : public QPushButton
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param brush pattern number.
   */
  BrushButton(int brush, QWidget* parent = 0);
  /** Destructor. */
  ~BrushButton();
  /**
   * Returns pattern number.
   * @return pattern number.
   */
  int value();
  /** Reimplemented from QPushButton. */
  virtual QSize sizeHint() const;

protected slots:
  /** Displays the dialog for selection of a pattern. */
  void slotBrush();

signals:
  /**
   * Emitted when brush changed.
   * @param filled true for brush pattern.
   */
  void valueChanged(bool filled);

protected:
  /** Reimplemented from QPushButton. */
  virtual void paintEvent(QPaintEvent* event);
  int brush;
};

/**
  * Brush radio button. Provides a radio button which shows a
  * brush fill pattern.
  * @author Werner Stille
  */
class BrushRadioButton : public QRadioButton
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param brush pattern number.
   * @param parent pointer to parent widget.
   */
  BrushRadioButton(int brush, QWidget* parent = 0);
  /** Destructor. */
  ~BrushRadioButton();

protected:
  /** Reimplemented from QRadioButton. */
  virtual void paintEvent(QPaintEvent* event);
  int brush;
};

/**
  * Brush dialog. Provides a dialog to select a brush fill pattern.
  * @author Werner Stille
  */
class BrushDlg : public KDialog
{
  Q_OBJECT
public: 
  /**
   * Constructor.
   * @param brush pointer to pattern number.
   * @param parent pointer to parent widget.
   */
  BrushDlg(int* brush, QWidget* parent = 0);
  /** Destructor. */
  ~BrushDlg();

protected slots:
  /** Reimplemented from KDialog. */
  virtual void slotButtonClicked(int button);
  /** Enables Ok button. */
  void slotEdited();

protected:
  int* brush;
  KButtonGroup* bg;
};

#endif
