/***************************************************************************
                          doublespinbox.cpp  -  description
                             -------------------
    begin                : Do Apr 14 2005
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlineedit.h>
#include <qvalidator.h>
#include "doubleedit.h"
#include "doublespinbox.h"

DoubleSpinBox::DoubleSpinBox(double val, double stp, QWidget* parent,
                             char format, int prc) :
 QSpinBox(parent), frm(format), prec(prc)
{
  setRange(qRound(val / stp) - 1000000000, qRound(val / stp) + 1000000000);
  init(val, stp);
}

DoubleSpinBox::DoubleSpinBox(double val, double stp, double minVal,
                             double maxVal, QWidget* parent, char format,
                             int prc) :
 QSpinBox(parent), frm(format), prec(prc)
{
  setRange(qRound(minVal / stp), qRound(maxVal / stp));
  init(val, stp);
}

DoubleSpinBox::~DoubleSpinBox()
{
}

double DoubleSpinBox::value() const
{
  return DoubleEdit::readNumber(text());
}

void DoubleSpinBox::init(double val, double stp)
{
  lineEdit()->setValidator(new QDoubleValidator(minimum() * stp,
                                                maximum() * stp, prec, this));
  step = QString::number(stp, 'g', 2).toDouble();
  QSpinBox::setValue(qRound(val / step));
}

QString DoubleSpinBox::textFromValue(int value) const
{
  return DoubleEdit::formatNumber(value * step, frm, prec);
}

int DoubleSpinBox::valueFromText(const QString& text) const
{
  return qRound(DoubleEdit::readNumber(text) / step);
}
