/***************************************************************************
                          doublespinbox.h  -  description
                             -------------------
    begin                : Do Apr 14 2005
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DOUBLESPINBOX_H
#define DOUBLESPINBOX_H

#include <QSpinBox>

/**
  * Double value spinbox class. Provides a double value mapped QSpinBox widget
  * with validator.
  * @author Werner Stille
  */

class DoubleSpinBox : public QSpinBox
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param val initial double value.
   * @param step step size.
   * @param parent pointer to parent widget.
   * @param format format character for value display.
   * @param prec precision (number of digits) for value display.
   */
  DoubleSpinBox(double val, double step, QWidget* parent = 0, char format = 'g',
                int prec = 6);
  /**
   * Constructor.
   * @param val initial double value.
   * @param step step size.
   * @param minVal minimum value.
   * @param maxVal maximum value.
   * @param parent pointer to parent widget.
   * @param format format character for value display.
   * @param prec precision (number of digits) for value display.
   */
  DoubleSpinBox(double val, double step, double minVal, double maxVal,
                QWidget* parent = 0, char format = 'f', int prec = 2);
  /** Destructor. */
  ~DoubleSpinBox();
  /**
   * Returns value.
   * @return value.
   */
  double value() const;

protected:
  /**
   * Initializes several elements.
   * @param val initial double value.
   * @param step step size.
   */
  void init(double val, double step);
  /** Reimplemented from QSpinBox. */
  virtual QString textFromValue(int value) const;
  /** Reimplemented from QSpinBox. */
  virtual int valueFromText(const QString& text) const;
  char frm;
  int prec;
  double step;
};

#endif
