/***************************************************************************
                          ellipseitem.cpp  -  description
                             -------------------
    begin                : Sun Apr 29 2001
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QBrush>
#include <KConfigGroup>
#include <KLocalizedString>
#include "ellipseitem.h"
#include "kplscene.h"

EllipseItem::EllipseItem(double x, double y, unsigned color,
                         QGraphicsItem* parent, double w, double h,
                         bool visible, int fill, double relSize,
                         double dir) :
 DecoItem(color, x, y, relSize), QGraphicsEllipseItem(parent),
 m_fillStyle(fill), m_width(w), m_height(h), m_dir(dir)

{
  if (!visible)
    hide();
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

EllipseItem::EllipseItem(const EllipseItem& a) :
 DecoItem(a), QGraphicsEllipseItem(), m_fillStyle(a.m_fillStyle),
 m_width(a.m_width), m_height(a.m_height), m_dir(a.m_dir)
{
  setZValue(a.zValue());
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
  setSelected(a.isSelected());
  setVisible(a.isVisible());
}

EllipseItem::EllipseItem(KConfigGroup* plo, unsigned color,
                         QGraphicsItem* parent) :
 DecoItem(plo, color), QGraphicsEllipseItem(parent)
{
  if (!plo->readEntry("active", true))
    hide();
  m_fillStyle = plo->readEntry("fillstyle", 0);
  if (!m_fillStyle)
    if (plo->readEntry("filled", false))
      m_fillStyle = 1;
  m_width = plo->readEntry("width", 3.0);
  m_height = plo->readEntry("height", 2.0);
  m_dir = plo->readEntry("dir", 0.0);
  setFlags(QGraphicsItem::ItemIsMovable | QGraphicsItem::ItemIsSelectable);
}

EllipseItem::~EllipseItem()
{
}

QPixmap EllipseItem::pixmap() const
{
  return KplItem::pixmap(QPen(m_color), m_fillStyle ? -11 : 0);
}

void EllipseItem::setPenWidth()
{
  if (!m_fillStyle) {
    QPen p = pen();
    double w = penWidth(this);
    if (p.widthF() != w) {
      prepareGeometryChange();
      setPen(changePenWidth(p, w));
    }
  }
}

void EllipseItem::setProperties(const QTransform& m)
{
  m_matrix = m;
  setProperties();
}

void EllipseItem::update()
{
  prepareGeometryChange();
  setProperties();
  QGraphicsItem::update();
}

QString EllipseItem::settings() const
{
  return QString("x = %1 cm, y = %2 cm, ").arg(m_x0).arg(m_y0) + i18n("size") +
    QString(" %1 cm x %2 cm").arg(m_width).arg(m_height);
}

void EllipseItem::writePlo(KConfigGroup* plo) const
{
  plo->writeEntry("Type", "ELLIPSEITEM");
  DecoItem::writePlo(plo, isVisible());
  plo->writeEntry("fillstyle", m_fillStyle);
  plo->writeEntry("width", m_width);
  plo->writeEntry("height", m_height);
  if (m_dir)
    plo->writeEntry("dir", m_dir);
}

void EllipseItem::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
  QGraphicsEllipseItem::mouseReleaseEvent(event);
  ((KplScene*)scene())->update(this);
}

void EllipseItem::setProperties()
{
  double w = m_matrix.m11() * m_width;
  double h = qAbs(m_matrix.m22() * m_height);
  setRect(-0.5 * w, -0.5 * h, w, h);
  setPos(m_matrix.map(QPointF(m_x0, m_y0)));
#if QT_VERSION < 0x040600
  setTransform(QTransform().rotate(-m_dir));
#else
  setRotation(-m_dir);
#endif
  if (m_fillStyle) {
    setBrush(createBrush(m_color, m_fillStyle));
    setPen(Qt::NoPen);
  } else
    setPen(createPen(this, m_color));
}
