/***************************************************************************
                          fitdlg.h  -  description
                             -------------------
    begin                : Tue May 25 1999
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FITDLG_H
#define FITDLG_H

#include <KDialog>
#include "funitem.h"

class QCheckBox;
class QLabel;
class QListWidget;
class ArrayItem;
class DoubleEdit;
class KplDoc;
class LMFit;
class SourceWidget;
class SpinBox;

/**
  * Correlation matrix dialog class. Provides a dialog for displaying
  * the parameter correlation matrix.
  *@author Werner Stille
  */
class CorrDlg : public KDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param np number of fitted parameters.
   * @param ip array with parameter indices.
   * @param corr np * np matrix with correlation coefficients.
   */
  CorrDlg(QWidget* parent, int np, const int* ip, const double* corr);
  /** Destructor. */
  ~CorrDlg();
};

/**
  * Fit dialog class. Provides a dialog for performing a parameter fit
  * of an ordinary function to a data set of an array.
  *@author Werner Stille
  */
class FitDlg : public KDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param ad pointer to array item list.
   * @param fd function item list.
   * @param mode combination of ShowDlg and AutoFit.
   */
  FitDlg(QWidget* parent, KplDoc* model, const QList<ArrayItem*>* ad,
         const QList<FunItem*>& fd, int mode);
  /** Destructor. */
  ~FitDlg();
  /** Dialog mode flags. */
  enum DlgMode{ShowDlg = 1, AutoFit = 2, SavePar = 4, Follow = 8};
  /** Applies settings. */
  void apply();
  /** Performs Levenberg-Marquardt fit. */
  void fit();

protected:
  bool running;
  int ip[FunItem::Kpl_npMax];
  int np, dlgMode;
  double corr[FunItem::Kpl_npMax * FunItem::Kpl_npMax];
  KplDoc* m;
  QVector<double> fvec, sig;
  const QList<ArrayItem*>* ai;
  QList<FunItem*> fi;
  QList<QWidget*> widgetList;
  LMFit* lm;
  QListWidget* results;
  DoubleEdit* eTol;
  DoubleEdit* par[FunItem::Kpl_npMax];
  QLabel* lArr;
  QLabel* epar[FunItem::Kpl_npMax];
  QCheckBox* enFit[FunItem::Kpl_npMax];
  QCheckBox *errCol, *nonLin;
  SpinBox *sArr, *sItmax;
  QPushButton* showCorr;
  QPushButton* errMod;
  FunItem::DataErrorStruct* err;
  SourceWidget* sw;

protected slots:
  /**
   * Disables/enables error model button.
   * @param state error column state, disables button when true.
   */
  void errColToggled(bool state);
  /**
   * Disables/enables widgets for nonlinear fits.
   * @param on enables widgets when true.
   */
  void enableNonLin(bool on);
  /** Reimplemented from KDialog. */
  virtual void slotButtonClicked(int button);
  /** Displays parameter correlation matrix. */
  void slotCorr();
  /** Displays the dialog for setting the calculation of error bars. */
  void slotErrMod();
  /** Displays the dialog for selection of a parameter file to read. */
  void slotGetParameter();
  /**
   * Displays message line in listbox.
   * @param msg message text.
   */
  void slotMessage(const QString& msg);
  /** Displays the dialog for selection of a parameter file to save. */
  void slotSaveParameter();
  /** Updates display for selected array in data error settings. */
  void updArray();
};

#endif
