/***************************************************************************
                          framedlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QRadioButton>
#include <QToolButton>
#include <KButtonGroup>
#include <KColorButton>
#include <KLocalizedString>
#include <KUndoStack>
#include "arrayitem.h"
#include "edittextdlg.h"
#include "framedlg.h"
#include "frameitem.h"
#include "funitem.h"
#include "kpldoc.h"
#include "kplsettings.h"
#include "doubleedit.h"
#include "spinbox.h"
#include "splineitem.h"

FrameDlg::FrameDlg(QWidget* parent, KplDoc* model, FrameItem* fi0,
                   QPoint pos) :
 KDialog(parent), m(model), norm(false), fi(fi0)
{
  setCaption(i18n("Frame"));
  setButtons(Help | Ok | Apply | Cancel);
  setButtonFocus(Ok);
  QWidget* page = new QWidget;
  setMainWidget(page);
  QGridLayout* grid = new QGridLayout(page);
  grid->setSpacing(spacingHint());
  grid->addWidget(bg = new KButtonGroup, 0, 0);
  bg->setTitle(i18n("Axis mode"));
  QVBoxLayout* vbox = new QVBoxLayout(bg);
  vbox->addWidget(new QRadioButton(i18n("Frame only")));
  vbox->addWidget(new QRadioButton(i18n("Tics")));
  vbox->addWidget(new QRadioButton(i18n("Tics with numbers")));
  vbox->addWidget(new QRadioButton(i18n("Grid")));
  vbox->addWidget(new QRadioButton(i18n("Grid with numbers")));
  bg->setSelected(fi->gridMode());
  grid->addLayout(vbox = new QVBoxLayout, 0, 1);
  QGroupBox* g;
  vbox->addWidget(g = new QGroupBox(i18n("Window")));
  QGridLayout* grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel(i18n("Left margin")), 0, 0);
  grid2->addWidget(ex0 = new QDoubleSpinBox, 0, 1);
  ex0->setMaximum(200);
  ex0->setSingleStep(0.1);
  ex0->setSuffix(" cm");
  ex0->setValue(fi->x0());
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid2->addWidget(new QLabel(i18n("Width")), 0, 3);
  grid2->addWidget(exl = new QDoubleSpinBox, 0, 4);
  exl->setRange(0.5, 200);
  exl->setSingleStep(0.1);
  exl->setSuffix(" cm");
  exl->setValue(fi->width());
  grid2->addWidget(new QLabel(i18n("Bottom margin")), 1, 0);
  grid2->addWidget(ey0 = new QDoubleSpinBox, 1, 1);
  ey0->setMaximum(200);
  ey0->setSingleStep(0.1);
  ey0->setSuffix(" cm");
  ey0->setValue(fi->y0());
  grid2->addWidget(new QLabel(i18n("Height")), 1, 3);
  grid2->addWidget(eyl = new QDoubleSpinBox, 1, 4);
  eyl->setRange(0.5, 200);
  eyl->setSingleStep(0.1);
  eyl->setSuffix(" cm");
  eyl->setValue(fi->height());
  vbox->addWidget(g = new QGroupBox(i18n("Scaling")));
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("xmin"), 0, 0);
  bool lx = fi->logX();
  grid2->addWidget(exMin = new DoubleEdit(fi->xMin(),
                                          lx ? 1.0e-300 : -1.0e300,
                                          1.0e300, 'g', 6), 0, 1);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid2->addWidget(new QLabel("xmax"), 0, 3);
  grid2->addWidget(exMax = new DoubleEdit(fi->xMax(),
                                          lx ? 1.0e-300 : -1.0e300,
                                          1.0e300, 'g', 6), 0, 4);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 5);
  grid2->addWidget(logX = new QCheckBox("log"), 0, 6);
  logX->setChecked(lx);
  grid2->addWidget(new QLabel("ymin"), 1, 0);
  bool ly = fi->logY();
  grid2->addWidget(eyMin = new DoubleEdit(fi->yMin(),
                                          ly ? 1.0e-300 : -1.0e300,
                                          1.0e300, 'g', 6), 1, 1);
  grid2->addWidget(new QLabel("ymax"), 1, 3);
  grid2->addWidget(eyMax = new DoubleEdit(fi->yMax(),
                                          ly ? 1.0e-300 : -1.0e300,
                                          1.0e300, 'g', 6), 1, 4);
  grid2->addWidget(logY = new QCheckBox("log"), 1, 6);
  logY->setChecked(ly);
  grid->addWidget(g = new QGroupBox(i18n("Colors")), 1, 0);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel(i18n("Frame")), 0, 0);
  grid2->addWidget(colFrame = new KColorButton(QColor(fi->colFrame())), 0, 1);
  grid2->addWidget(new QLabel(i18n("Grid")), 1, 0);
  grid2->addWidget(colGrid = new KColorButton(QColor(fi->colGrid())), 1, 1);
  grid->addWidget(g = new QGroupBox(i18n("Axes")), 1, 1);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("xtic"), 0, 0);
  grid2->addWidget(exTic = new DoubleEdit(fi->xTic()), 0, 1);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid2->addWidget(new QLabel("xoff"), 0, 3);
  grid2->addWidget(exOff = new DoubleEdit(fi->xOff(), 0, 1.0e300, 'g', 6), 0,
                   4);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 5);
  grid2->addWidget(new QLabel("mticx"), 0, 6);
  grid2->addWidget(emTicx = new SpinBox(fi->minTicX(), 0, 10), 0, 7);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 8);
  grid2->addWidget(new QLabel("ndigx"), 0, 9);
  grid2->addWidget(enDigx = new SpinBox(fi->numDigX(), -1, 8), 0, 10);
  enDigx->setEnabled(!lx);
  grid2->addWidget(new QLabel("ytic"), 1, 0);
  grid2->addWidget(eyTic = new DoubleEdit(fi->yTic()), 1, 1);
  grid2->addWidget(new QLabel("yoff"), 1, 3);
  grid2->addWidget(eyOff = new DoubleEdit(fi->yOff(), 0, 1.0e300, 'g', 6), 1,
                   4);
  grid2->addWidget(new QLabel("mticy"), 1, 6);
  grid2->addWidget(emTicy = new SpinBox(fi->minTicY(), 0, 10), 1, 7);
  grid2->addWidget(new QLabel("ndigy"), 1, 9);
  grid2->addWidget(enDigy = new SpinBox(fi->numDigY(), -1, 8), 1, 10);
  enDigy->setEnabled(!ly);
  QVBoxLayout* vbox2 = new QVBoxLayout;
  grid->addLayout(vbox2, 2, 0);
  vbox2->addWidget(g = new QGroupBox(i18n("Automatic")));
  QVBoxLayout* vbox3 = new QVBoxLayout(g);
  vbox3->addWidget(autoNorm = new QCheckBox(i18n("Normalization")));
  autoNorm->setChecked(KplSettings::normalization() &&
                       (!(lx || ly)));
  QPushButton* b = new QPushButton(i18n("Autoscale"));
  vbox3->addWidget(b);
  connect(b, SIGNAL(clicked()), SLOT(slotAutoScale()));
  bool e = false;
  foreach (QGraphicsItem* item, fi->childItems())
    if (item->isVisible()) {
      switch (item->type()) {
        case ArrayItem::Type:
        case FunItem::Type:
        case ParFunItem::Type:
        case SplineItem::Type:
          e = true;
          break;
        default:
          break;
      }
      if (e)
        break;
    }
  b->setEnabled(e);
  autoNorm->setEnabled(e);
  grid->addWidget(g = new QGroupBox(i18n("Labels")), 2, 1);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("iex"), 0, 0);
  grid2->addWidget(ex = new SpinBox(fi->expX(), -300, 300), 0, 1);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::Minimum), 0, 2);
  grid2->addWidget(new QLabel(i18n("x axis")), 0, 3);
  grid2->addWidget(eLetX = new QLineEdit(fi->textX()), 0, 4);
  QToolButton* tb;
  grid2->addWidget(tb = new QToolButton, 0, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetX()));
  grid2->addWidget(xBottom = new QCheckBox(i18n("Bottom")), 0, 6);
  xBottom->setChecked(fi->xBottom());
  grid2->addWidget(xTop = new QCheckBox(i18n("Top")), 0, 7);
  xTop->setChecked(fi->xTop());
  grid2->addWidget(new QLabel("iey"), 1, 0);
  grid2->addWidget(ey = new SpinBox(fi->expY(), -300, 300), 1, 1);
  grid2->addWidget(new QLabel(i18n("y axis")), 1, 3);
  grid2->addWidget(eLetY = new QLineEdit(fi->textY()), 1, 4);
  grid2->addWidget(tb = new QToolButton, 1, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetY()));
  grid2->addWidget(yLeft = new QCheckBox(i18n("Left")), 1, 6);
  yLeft->setChecked(fi->yLeft());
  grid2->addWidget(yRight = new QCheckBox(i18n("Right")), 1, 7);
  yRight->setChecked(fi->yRight());
  grid2->addWidget(new QLabel(i18n("Heading")), 2, 3);
  grid2->addWidget(eLetH = new QLineEdit(fi->textH()), 2, 4);
  grid2->addWidget(tb = new QToolButton, 2, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetH()));
  QHBoxLayout* hbox = new QHBoxLayout;
  vbox2->addLayout(hbox);
  hbox->addWidget(new QLabel(i18n("Symbol size"), page));
  hbox->addWidget(eRelSiz = new SpinBox(qRound(100 * fi->relSize()), 10, 800));
  eRelSiz->setSuffix(" %");
  enableButtonApply(false);
  enableButtonOk(false);
  setHelp("SEC-FRAME");
  restoreDialogSize(KGlobal::config()->group("FrameDlg"));
  move(pos);
  connect(bg, SIGNAL(changed(int)), SLOT(slotEdited()));
  connect(logX, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(logY, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(autoNorm, SIGNAL(toggled(bool)), SLOT(slotNormToggled(bool)));
  connect(ex0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(exl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(ey0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eyl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(exMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exOff, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyOff, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eLetX, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetY, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetH, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(emTicx, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(emTicy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigx, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ex, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ey, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(eRelSiz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(xBottom, SIGNAL(toggled(bool)), SLOT(slotEdited()));
  connect(xTop, SIGNAL(toggled(bool)), SLOT(slotEdited()));
  connect(yLeft, SIGNAL(toggled(bool)), SLOT(slotEdited()));
  connect(yRight, SIGNAL(toggled(bool)), SLOT(slotEdited()));
  connect(colFrame, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(colGrid, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
}

FrameDlg::~FrameDlg()
{
  KConfigGroup config = KGlobal::config()->group("FrameDlg");
  saveDialogSize(config);
}

void FrameDlg::normalize() const
{
  foreach (QGraphicsItem* item, fi->childItems())
    if (item->isVisible()) {
      QGraphicsItem* newItem = 0;
      bool b = true;
      switch (item->type()) {
        case ArrayItem::Type: {
          ArrayItem* it = qgraphicsitem_cast<ArrayItem*>(item);
          b = it->isNormalized(fx, fy);
          if (!b) {
            ArrayItem* itNew = new ArrayItem(*it);
            itNew->normalize(fx, fy);
            newItem = itNew;
          }
          break;
        }
        case FunItem::Type: {
          FunItem* it = qgraphicsitem_cast<FunItem*>(item);
          b = it->isNormalized(fx, fy);
          if (!b) {
            FunItem* itNew = new FunItem(*it);
            itNew->normalize(fx, fy);
            newItem = itNew;
          }
          break;
        }
        case ParFunItem::Type: {
          ParFunItem* it = qgraphicsitem_cast<ParFunItem*>(item);
          b = it->isNormalized(fx, fy);
          if (!b) {
            ParFunItem* itNew = new ParFunItem(*it);
            itNew->normalize(fx, fy);
            newItem = itNew;
          }
          break;
        }
        case SplineItem::Type: {
          SplineItem* it = qgraphicsitem_cast<SplineItem*>(item);
          b = it->isNormalized(fx, fy);
          if (!b) {
            SplineItem* itNew = new SplineItem(*it);
            itNew->normalize(fx, fy);
            newItem = itNew;
          }
          break;
        }
      }
      if (newItem)
        new EditCommand(newItem, item, m, macro);
    }
}

void FrameDlg::slotAutoScale()
{
  FrameItem ft = *fi;
  ft.setWidth(exl->value());
  ft.setHeight(eyl->value());
  ft.setLogX(logX->isChecked());
  ft.setLogY(logY->isChecked());
  norm = ft.calcAutoScale(autoNorm->isChecked(), fi->childItems(), &fx, &fy);
  if (norm) {
    ex->setValue(ft.expX());
    emTicx->setValue(ft.minTicX());
    enDigx->setValue(ft.numDigX());
    exMin->setValue(ft.xMin());
    exMax->setValue(ft.xMax());
    exTic->setValue(ft.xTic());
    ey->setValue(ft.expY());
    emTicy->setValue(ft.minTicY());
    enDigy->setValue(ft.numDigY());
    eyMin->setValue(ft.yMin());
    eyMax->setValue(ft.yMax());
    eyTic->setValue(ft.yTic());
    exOff->setValue(0);
    eyOff->setValue(0);
    slotEdited();
  }
}

void FrameDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      FrameItem* fiNew = new FrameItem(*fi);
      double xmin = exMin->value();
      fiNew->setXMin(xmin);
      double xmax = exMax->value();
      fiNew->setXMax(xmax);
      double ymin = eyMin->value();
      fiNew->setYMin(ymin);
      double ymax = eyMax->value();
      fiNew->setYMax(ymax);
      bool lx = logX->isChecked();
      fiNew->setLogX(lx);
      bool ly = logY->isChecked();
      fiNew->setLogY(ly);
      fiNew->setXBottom(xBottom->isChecked());
      fiNew->setXTop(xTop->isChecked());
      fiNew->setYLeft(yLeft->isChecked());
      fiNew->setYRight(yRight->isChecked());
      double xtic = exTic->value();
      if (!lx) {
        double d = 0.01 *(xmax - xmin);
        if (xtic < d) {
          exTic->setValue(d);
          xtic = exTic->value();
        }
      }
      fiNew->setXTic(xtic);
      double ytic = eyTic->value();
      if (!ly) {
        double d = 0.01 *(ymax - ymin);
        if (ytic < d) {
          eyTic->setValue(d);
          ytic = eyTic->value();
        }
      }
      fiNew->setYTic(ytic);
      fiNew->setXOff(exOff->value());
      fiNew->setYOff(eyOff->value());
      fiNew->setX0(ex0->value());
      fiNew->setY0(ey0->value());
      fiNew->setWidth(exl->value());
      fiNew->setHeight(eyl->value());
      fiNew->setColFrame(colFrame->color().rgb());
      fiNew->setColGrid(colGrid->color().rgb());
      fiNew->setExpX(ex->value());
      fiNew->setExpY(ey->value());
      fiNew->setMinTicX(emTicx->value());
      fiNew->setMinTicY(emTicy->value());
      fiNew->setNumDigX(enDigx->value());
      fiNew->setNumDigY(enDigy->value());
      fiNew->setTextX(eLetX->text());
      fiNew->setTextY(eLetY->text());
      fiNew->setTextH(eLetH->text());
      fiNew->setRelSize(0.01 * eRelSiz->value());
      fiNew->setGridMode(bg->selected());
      macro = new QUndoCommand();
      if (norm)
        normalize();
      new EditCommand(fiNew, fi, m, macro);
      m->undoStack()->push(macro);
      fi = fiNew;
      if (button == Ok)
        accept();
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}

void FrameDlg::slotEdited()
{
  enableButtonApply(true);
  enableButtonOk(true);
}

void FrameDlg::slotLogToggled(bool state)
{
  const QCheckBox* b = (const QCheckBox*) sender();
  DoubleEdit *mi, *ma, *off = 0;
  if ((b == logX) || (b == logY)) {
    SpinBox* nd;
    if (b == logX) {
      nd = enDigx;
      mi = exMin;
      ma = exMax;
      off = exOff;
    } else {
      nd = enDigy;
      mi = eyMin;
      ma = eyMax;
      off = eyOff;
    }
    nd->setEnabled(!state);
    off->setEnabled(!state);
    double bottom = state ? 1.0e-300 : -1.0e300;
    mi->setBottom(bottom);
    ma->setBottom(bottom);
  }
  if (state) {
    autoNorm->setChecked(false);
    off->setText("0");
  }
  slotEdited();
}

void FrameDlg::slotNormToggled(bool state)
{
  if (state) {
    logX->setChecked(false);
    logY->setChecked(false);
  }
}

void FrameDlg::slotEditLetH()
{
  EditTextDlg::editText(this, eLetH);
  setButtonFocus(Ok);
}

void FrameDlg::slotEditLetX()
{
  EditTextDlg::editText(this, eLetX);
  setButtonFocus(Ok);
}

void FrameDlg::slotEditLetY()
{
  EditTextDlg::editText(this, eLetY);
  setButtonFocus(Ok);
}
