/***************************************************************************
                          funcdlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FUNCDLG_H
#define FUNCDLG_H

#include <KDialog>

class QLineEdit;
class KColorButton;
class BrushButton;
class FunItem;
class KplDoc;
class DoubleEdit;
class DoubleSpinBox;
class SpinBox;
class ParFunItem;

/**
  * Function settings dialog class. Provides a dialog for the settings
  * of an ordinary function.
  * @author Werner Stille
  */
class FuncDlg : public KDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param fi pointer to function object.
   * @param pos dialog position.
   */
  FuncDlg(QWidget* parent, KplDoc* model, FunItem* fi, QPoint pos);
  /** Destructor. */
  ~FuncDlg();
  /**
   * Opens a function selection dialog.
   * @param parent parent widget.
   * @param name QLineEdit widget displaying the library filename.
   * @param func QLineEdit widget to display the function name.
   */
  static void getFunc(QWidget* parent, QLineEdit* name, QLineEdit* func);

protected:
  /**
   * Copies settings to temporary function object.
   * @return false for error.
   */
  bool getTemp();
  KplDoc* m;
  FunItem *fi, *fit;
  QLineEdit *fileName, *func;
  BrushButton* fill;
  DoubleEdit *exMin, *exMax, *exStep, *efx, *efy;
  DoubleSpinBox *ex0, *ey0;
  SpinBox *symb, *eRelSize;
  KColorButton* colData;

protected slots:
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /** Reimplemented from KDialog. */
  virtual void slotButtonClicked(int button);
  /** Enables buttons. */
  void slotEdited();
  /** Displays the dialog for parameter editing. */
  void slotEditParameter();
  /** Displays the dialog for selection of a library file to load. */
  void slotFile();
  /** Displays the dialog for selection of a function. */
  void slotFunc();
  /** Displays the dialog for selection of a parameter file to read. */
  void slotGetParameter();
  /** Displays the dialog for selection of a parameter file to save. */
  void slotSaveParameter();
};

/**
  * Parametric function settings dialog class. Provides a dialog for the settings
  * of parametric functions.
  * @author Werner Stille
  */
class ParFuncDlg : public KDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param fi pointer to parametric function object.
   * @param pos dialog position.
   */
  ParFuncDlg(QWidget* parent, KplDoc* model, ParFunItem* fi, QPoint pos);
  /** Destructor. */
  ~ParFuncDlg();

protected:
  /**
   * Copies settings to temporary parametric function object.
   * @return false for error.
   */
  bool getTemp();
  KplDoc* m;
  ParFunItem *fi, *fit;
  QLineEdit *fileNamex, *funcx, *fileNamey, *funcy;
  BrushButton* fill;
  DoubleEdit *etMin, *etMax, *etStep, *efx, *efy;
  DoubleSpinBox *ex0, *ey0;
  SpinBox *symb, *eRelSize;
  KColorButton* colData;

protected slots:
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /** Reimplemented from KDialog. */
  virtual void slotButtonClicked(int button);
  /** Enables buttons. */
  void slotEdited();
  /** Displays the dialog for parameter editing for the x axis function. */
  void slotEditParameterX();
  /** Displays the dialog for parameter editing for the y axis function. */
  void slotEditParameterY();
  /** Displays the dialog for selection of a library file to load for the x axis. */
  void slotFileX();
  /** Displays the dialog for selection of a library file to load for the y axis. */
  void slotFileY();
  /** Displays the dialog for selection of a function for the x axis. */
  void slotFuncX();
  /** Displays the dialog for selection of a function for the y axis. */
  void slotFuncY();
  /** Displays the dialog for selection of a parameter file to read for the x axis function. */
  void slotGetParameterX();
  /** Displays the dialog for selection of a parameter file to read for the y axis function. */
  void slotGetParameterY();
  /** Displays the dialog for selection of a parameter file to save for the x axis function. */
  void slotSaveParameterX();
  /** Displays the dialog for selection of a parameter file to save for the y axis function. */
  void slotSaveParameterY();
};

#endif
