/***************************************************************************
                          kplview.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999

    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QDragEnterEvent>
#include <QRubberBand>
#include <QScrollBar>
#include <KUrl>
#include "kplview.h"

KplView::KplView(double f, QWidget* parent) :
 QGraphicsView(parent), m_zoom(f), rubberBand(0)
{
  setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
  scale(f, f);
  setContextMenuPolicy(Qt::CustomContextMenu);
  setMouseTracking(true);
  viewport()->setCursor(Qt::CrossCursor);
  setAlignment(Qt::AlignLeft | Qt::AlignBottom);
}

KplView::~KplView()
{
}

void KplView::dragEnterEvent(QDragEnterEvent* e)
{
  if (e->mimeData()->hasUrls())
    e->acceptProposedAction();
}

void KplView::dragMoveEvent(QDragMoveEvent*)
{
}

void KplView::dropEvent(QDropEvent* e)
{
  KUrl::List list;
  foreach (QUrl u, e->mimeData()->urls())
    if (!u.toString().isEmpty())
      list << u;
  emit urlsDropped(&list);
  e->acceptProposedAction();
}

void KplView::keyPressEvent(QKeyEvent* e)
{
  if (e == QKeySequence::MoveToStartOfLine)
    verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
  else if (e == QKeySequence::MoveToEndOfLine)
    verticalScrollBar()->triggerAction(QAbstractSlider::SliderToMinimum);
  else if (e == QKeySequence::MoveToStartOfDocument)
    horizontalScrollBar()->triggerAction(QAbstractSlider::SliderToMinimum);
  else if (e == QKeySequence::MoveToEndOfDocument)
    horizontalScrollBar()->triggerAction(QAbstractSlider::SliderToMaximum);
  else if (e == QKeySequence::SelectPreviousPage)
    horizontalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepSub);
  else if (e == QKeySequence::SelectNextPage)
    horizontalScrollBar()->triggerAction(QAbstractSlider::SliderPageStepAdd);
  else
    QGraphicsView::keyPressEvent(e);
}

void KplView::leaveEvent(QEvent* event)
{
  QWidget::leaveEvent(event);
  emit mouseLeaved();
}

void KplView::mouseMoveEvent(QMouseEvent* e)
{
  if (e->buttons() & Qt::MidButton)
    rubberBand->setGeometry(QRect(startPos, e->pos()).normalized());
  QGraphicsView::mouseMoveEvent(e);
}

void KplView::mousePressEvent(QMouseEvent* e)
{
  if (e->button() == Qt::MidButton) {
    startPos = e->pos();
    if (!rubberBand)
      rubberBand = new QRubberBand(QRubberBand::Rectangle, this);
    rubberBand->setGeometry(QRect(startPos, QSize()));
    rubberBand->show();
    return;
  }
  QGraphicsView::mousePressEvent(e);
}

void KplView::mouseReleaseEvent(QMouseEvent* e)
{
  if (e->button() == Qt::MidButton) {
    QRect r = rubberBand->geometry();
    if (r.isValid())
      emit zoomed(mapToScene(r.topLeft()), mapToScene(r.bottomRight()));
    rubberBand->hide();
    return;
  }
  QGraphicsView::mouseReleaseEvent(e);
}

void KplView::setZoom(double f)
{
  setTransform(QTransform());
  m_zoom = f;
  scale(f, f);
};
