/***************************************************************************
                          kplview.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999

    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KPLVIEW_H
#define KPLVIEW_H

#include <QGraphicsView>
#include <KUrl>

class QRubberBand;

/**
  * Kpl main view class.
  * Provides methods for painting and erasing the page widget and generating PostScript files.
  * @author Werner Stille
  */
class KplView : public QGraphicsView
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param f zoom factor.
   */
  KplView(double f = 1, QWidget* parent = 0);
  /** Destructor. */
  virtual ~KplView();
  /**
   * Sets zoom factor.
   * @param f zoom factor.
   */
  void setZoom(double f);
  /** Returns zoom factor. */
  inline double zoom() const { return m_zoom; };

signals:
  void mouseLeaved();
  /**
   * Emitted when Qt drop results in a list or URLs.
   * @param list pointer to list.
   */
  void urlsDropped(KUrl::List* list);
  /**
   * Emitted when plot should be zoomed.
   * @param p1 top left coordinates of new plot range in pixel units.
   * @param p2 bottom right coordinates of new plot range in pixel units.
   */
  void zoomed(const QPointF& p1, const QPointF& p2);

protected:
  /** Reimplemented from QGraphicsView. */
  virtual void dragEnterEvent(QDragEnterEvent* e);
  /** Reimplemented from QGraphicsView. */
  virtual void dragMoveEvent(QDragMoveEvent* e);
  /** Reimplemented from QGraphicsView. */
  virtual void dropEvent(QDropEvent* e);
  /** Reimplemented from QGraphicsView. */
  virtual void keyPressEvent(QKeyEvent* e);
  /** Reimplemented from QWidget. */
  virtual void leaveEvent(QEvent * event);
  /** Reimplemented from QGraphicsView. */
  virtual void mouseMoveEvent(QMouseEvent* e);
  /** Reimplemented from QGraphicsView. */
  virtual void mousePressEvent(QMouseEvent* e);
  /** Reimplemented from QGraphicsView. */
  virtual void mouseReleaseEvent(QMouseEvent* e);
  double m_zoom;
  QPoint startPos;
  QRubberBand* rubberBand;
};

#endif // KPLVIEW_H
