/***************************************************************************
                          legenddlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QDoubleSpinBox>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QLineEdit>
#include <QToolButton>
#include <KColorButton>
#include <KLocalizedString>
#include <KUndoStack>
#include "edittextdlg.h"
#include "kpldoc.h"
#include "legenddlg.h"
#include "legenditem.h"
#include "spinbox.h"
#include "symboldlg.h"

LegendDlg::LegendDlg(QWidget* parent, KplDoc* model, LegendItem* li0,
                     QPoint pos) :
 KDialog(parent), m(model), li(li0)
{
  setCaption(i18n("Legend"));
  setButtons(Help | Ok | Apply | Cancel);
  setButtonFocus(Ok);
  QWidget* page = new QWidget;
  setMainWidget(page);
  QVBoxLayout* vbox = new QVBoxLayout(page);
  vbox->setSpacing(spacingHint());
  QGroupBox* g = new QGroupBox(i18n("Data"));
  QGridLayout* grid = new QGridLayout;
  g->setLayout(grid);
  vbox->addWidget(g);
  grid->addWidget(new QLabel(i18n("Symbol")), 0, 0);
  grid->addWidget(symb = new SpinBox(li->symbol(), -17, 9), 0, 1);
  grid->addWidget(new SymbolButton(symb), 0, 2);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 3);
  grid->addWidget(new QLabel(i18n("Size")), 1, 0);
  grid->addWidget(eSymbSize = new SpinBox(qRound(100 * li->symbolSize()), 10,
                                          800), 1, 1);
  eSymbSize->setSuffix(" %");
  grid->addWidget(new QLabel(i18n("Color")), 1, 4);
  grid->addWidget(colSymb = new KColorButton(li->color()), 1, 5);
  grid->addWidget(new QLabel("x"), 2, 0);
  grid->addWidget(ex0 = new QDoubleSpinBox, 2, 1);
  ex0->setRange(-200, 200);
  ex0->setSingleStep(0.1);
  ex0->setSuffix(" cm");
  ex0->setValue(li->x0());
  grid->addWidget(new QLabel("y"), 2, 4);
  grid->addWidget(ey0 = new QDoubleSpinBox, 2, 5);
  ey0->setRange(-200, 200);
  ey0->setSingleStep(0.1);
  ey0->setSuffix(" cm");
  ey0->setValue(li->y0());
  vbox->addWidget(g = new QGroupBox(i18n("Label")));
  g->setLayout(grid = new QGridLayout);
  grid->addWidget(new QLabel(i18n("Text")), 0, 0);
  QHBoxLayout* hbox;
  grid->addLayout(hbox = new QHBoxLayout, 0, 1, 1, 4);
  hbox->addWidget(eText = new QLineEdit(li->text()));
  QToolButton* b = new QToolButton;
  b->setText("...");
  hbox->addWidget(b);
  connect(b, SIGNAL(clicked()), SLOT(slotEditText()));
  grid->addWidget(new QLabel(i18n("Size")), 1, 0);
  grid->addWidget(eRelSize = new SpinBox(qRound(100 * li->relSize()), 10, 800),
                  1, 1);
  eRelSize->setSuffix(" %");
  grid->addWidget(new QLabel(i18n("Color")), 1, 3);
  grid->addWidget(colText = new KColorButton(li->textColor()), 1, 4);
  grid->addWidget(new QLabel("xoff"), 2, 0);
  grid->addWidget(exl = new QDoubleSpinBox, 2, 1);
  exl->setRange(-200, 200);
  exl->setSingleStep(0.1);
  exl->setSuffix(" cm");
  exl->setValue(li->textX());
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 2, 2);
  grid->addWidget(new QLabel("yoff"), 2, 3);
  grid->addWidget(eyl = new QDoubleSpinBox, 2, 4);
  eyl->setRange(-200, 200);
  eyl->setSingleStep(0.1);
  eyl->setSuffix(" cm");
  eyl->setValue(li->textY());
  enableButtonApply(false);
  enableButtonOk(false);
  setHelp("SEC-LEGEND");
  resize(minimumSize());
  move(pos);
  connect(ex0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(ey0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(exl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eyl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(colSymb, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(colText, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(symb, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(eRelSize, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(eSymbSize, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(eText, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
}

LegendDlg::~LegendDlg()
{
}

void LegendDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      LegendItem* liNew = new LegendItem(*li);
      liNew->setX0(ex0->value());
      liNew->setY0(ey0->value());
      liNew->setTextX(exl->value());
      liNew->setTextY(eyl->value());
      liNew->setColor(colSymb->color().rgb());
      liNew->setTextColor(colText->color().rgb());
      liNew->setSymbol(symb->value());
      liNew->setSymbolSize(0.01 * eSymbSize->value());
      liNew->setRelSize(0.01 * eRelSize->value());
      liNew->setText(eText->text());
      m->undoStack()->push(new EditCommand(liNew, li, m));
      li = liNew;
      if (button == Ok)
        accept();
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}

void LegendDlg::slotEdited()
{
  enableButtonApply(true);
  enableButtonOk(true);
}

void LegendDlg::slotEditText()
{
  EditTextDlg dlg(this, eText);
  dlg.exec();
  setButtonFocus(Ok);
}
