/***************************************************************************
                          linedlg.cpp  -  description
                             -------------------
    begin                : Mon Apr 16 2001
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QLayout>
#include <QLabel>
#include <KLocalizedString>
#include <KUndoStack>
#include "arrowdlg.h"
#include "kpldoc.h"
#include "linedlg.h"
#include "lineitem.h"
#include "spinbox.h"
#include "symboldlg.h"

LineDlg::LineDlg(QWidget* parent, KplDoc* model, LineItem* ld0, QPoint pos) :
 DecoDlg(parent, i18n("Line"), "SEC-LINE", model, ld0, pos), li(ld0)
{
  ArrowDlg::initPolarEntries(li, grid, &eDir, &eLen);
  initEntries(1);
  lSize->setText(i18n("Width"));
  grid->addWidget(new QLabel(i18n("Type")), 3, 0);
  grid->addWidget(symb = new SpinBox(li->symbol(), 1, 9), 3, 1);
  symb->setValue(li->symbol());
  grid->addWidget(new SymbolButton(symb, true, false), 3, 2);
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eLen, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(symb, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
}

LineDlg::~LineDlg()
{
}

void LineDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      LineItem* liNew = new LineItem(*li);
      liNew->setSymbol(symb->value());
      liNew->setDir(eDir->value());
      liNew->setLen(eLen->value());
      getValues(liNew);
      m->undoStack()->push(new EditCommand(liNew, li, m));
      li = liNew;
      if (button == Ok)
        accept();
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}
