/***************************************************************************
                          lineitem.h  -  description
                             -------------------
    begin                : Mon Apr 16 2001
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LINEITEM_H
#define LINEITEM_H

#include <QGraphicsLineItem>
#include "kplitem.h"

class KConfigGroup;

/**
  * Line item class. Class for line decoration items.
  * @author Werner Stille
  */
class LineItem : public VectorItem, public QGraphicsLineItem
{
public:
  /**
    * Constructor. Initializes corresponding to arguments.
    * @param x x start position in cm.
    * @param y y start position in cm.
    * @param color color for line.
    * @param parent pointer to parent item.
    * @param len total length in cm.
    * @param visible true for visible item.
    * @param type line type.
    * @param relSize relative size of text.
    * @param dir direction in degree.
    */
  LineItem(double x = 0, double y = 0, unsigned color = 0, QGraphicsItem* parent = 0,
           double len = 1, bool visible = true, int type = 1,
           double relSize = 1, double dir = 0);
  /**
    * Copy constructor.
    * @param a source item.
    */
  LineItem(const LineItem& a);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KSimpleConfig object containing item data.
    * @param color default color for line.
    * @param parent pointer to parent item.
    */
  LineItem(KConfigGroup* plo, unsigned color, QGraphicsItem* parent = 0);
  /** Destructor. */
  virtual ~LineItem();
  enum {Type = UserType + Line};
  /** Returns decoration pixmap for item list. */
  QPixmap pixmap() const;
  /** Sets offset for dashed lines. */
  void setDashOffset();
  /** Sets pen width. */
  void setPenWidth();
  /**
    * Sets properties according to transformation matrix.
    * @param m transformation matrix.
    */
  void setProperties(const QTransform& m);
  /**
    * Sets line type.
    * @param type line type.
    */
  inline void setSymbol(int type) { m_symbol = type; };
  /** Returns settings text for item list. */
  QString settings() const;
  /** Returns line type. */
  inline int symbol() const { return m_symbol; };
  /** Reimplemented from QGraphicsItem. */
  virtual int type() const { return Type; };
  /** Applies changes and forces redrawing. */
  void update();
  /**
   * Writes into plot file.
   * @param plo KConfigGroup object.
   */
  void writePlo(KConfigGroup* plo) const;

protected:
  /** Reimplemented from QGraphicsItem. */
  virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* event);
  /** Sets properties according to transformation matrix. */
  void setProperties();
  int m_symbol;
};

#endif
