/***************************************************************************
                          rectdlg.cpp  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QLabel>
#include <QLayout>
#include <QLabel>
#include <KLocalizedString>
#include <KUndoStack>
#include "brushdlg.h"
#include "kpldoc.h"
#include "rectitem.h"
#include "rectdlg.h"
#include "spinbox.h"

RectDlg::RectDlg(QWidget* parent, KplDoc* model, RectItem* ri0, QPoint pos) :
 DecoDlg(parent, i18n("Rectangle"), "SEC-RECT", model, ri0, pos), ri(ri0)
{
  grid->addWidget(new QLabel(i18n("Width")), 0, 0);
  grid->addWidget(eWidth = new QDoubleSpinBox, 0, 1);
  eWidth->setRange(0.01, 200);
  eWidth->setSingleStep(0.1);
  eWidth->setSuffix(" cm");
  eWidth->setValue(ri->width());
  connect(eWidth, SIGNAL(valueChanged(const QString&)), SLOT(slotAspect()));
  grid->addWidget(new QLabel(i18n("Height")), 0, 3);
  grid->addWidget(eHeight = new QDoubleSpinBox, 0, 4);
  eHeight->setRange(0.01, 200);
  eHeight->setSingleStep(0.1);
  eHeight->setSuffix(" cm");
  eHeight->setValue(ri->height());
  grid->addWidget(isotropic = new QCheckBox(i18n("Square")), 1, 1, 1, 5);
  connect(isotropic, SIGNAL(toggled(bool)), SLOT(slotAspect()));
  isotropic->setChecked(ri->width() == ri->height());
  grid->addWidget(new QLabel(i18n("Rotation")), 2, 0);
  grid->addWidget(eDir = new QDoubleSpinBox, 2, 1);
  eDir->setRange(-360, 360);
  eDir->setSuffix("\260");
  eDir->setValue(ri->dir());
  initEntries(3);
  eRelSize->setDisabled(ri->fillStyle());
  lSize->setText(i18n("Line width"));
  grid->addWidget(new QLabel(i18n("Fill pattern")), 5, 3);
  grid->addWidget(fill = new BrushButton(ri->fillStyle()), 5, 4);
  connect(fill, SIGNAL(valueChanged(bool)), eRelSize, SLOT(setDisabled(bool)));
  connect(fill, SIGNAL(valueChanged(bool)), SLOT(slotEdited()));
  connect(eHeight, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eDir, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
}

RectDlg::~RectDlg()
{
}

void RectDlg::slotAspect()
{
  bool iso = isotropic->isChecked();
  if (iso)
    eHeight->setValue(eWidth->value());
  eHeight->setDisabled(iso);
  slotEdited();
}

void RectDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      RectItem* riNew = new RectItem(*ri);
      getValues(riNew);
      double w = eWidth->value();
      riNew->setWidth(w);
      riNew->setHeight(isotropic->isChecked() ? w : eHeight->value());
      riNew->setDir(eDir->value());
      riNew->setFillStyle(fill->value());
      m->undoStack()->push(new EditCommand(riNew, ri, m));
      ri = riNew;
      if (button == Ok)
        accept();
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}
