/***************************************************************************
                          rectitem.h  -  description
                             -------------------
    begin                : Sat Apr 28 2001
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef RECTITEM_H
#define RECTITEM_H

#include <QGraphicsRectItem>
#include "kplitem.h"

/**
  * Rectangle item class. Class for rectangle decoration items.
  * @author Werner Stille
  */
class RectItem : public DecoItem, public QGraphicsRectItem
{
public:
  /**
    * Constructor. Initializes corresponding to arguments.
    * @param x x position in cm.
    * @param y y position in cm.
    * @param color color for rectangle.
    * @param parent pointer to parent item.
    * @param w width in cm.
    * @param h height in cm.
    * @param visible true for visible item.
    * @param fillStyle > 0 for pattern filled rectangle.
    * @param relSize relative line width.
    * @param dir rotation angle in degree.
    */
  RectItem(double x = 0, double y = 0, unsigned color = 0,
           QGraphicsItem* parent = 0, double w = 3, double h = 2,
           bool visible = true, int fillStyle = 0, double relSize = 1,
           double dir = 0);
  /**
    * Copy constructor.
    * @param a source item.
    */
  RectItem(const RectItem& a);
  /**
    * Constructor. Initializes corresponding to plot file.
    * @param plo pointer to KConfigGroup object containing item data.
    * @param color default color for rectangle.
    * @param parent pointer to parent item.
    */
 RectItem(KConfigGroup* plo, unsigned color, QGraphicsItem* parent = 0);
  /** Destructor. */
  virtual ~RectItem();
  enum {Type = UserType + Rectangle};
  /** Returns rotation angle in deg. */
  inline double dir() const { return m_dir; };
  /** Returns fill pattern number > 0 for pattern filled ellipse. */
  inline int fillStyle() const { return m_fillStyle; };
  /** Returns height in cm. */
  inline double height() const { return m_height; };
  /** Returns decoration pixmap for item list. */
  QPixmap pixmap() const;
  /**
    * Sets rotation angle.
    * @param dir rotation angle in deg.
    */
  inline void setDir(double dir) { m_dir = dir; };
  /**
    * Sets fill style.
    * @param style fill pattern number > 0 for pattern filled ellipse.
    */
  inline void setFillStyle( int style) { m_fillStyle = style; };
  /**
    * Sets height.
    * @param h height in cm.
    */
  inline void setHeight( double h) { m_height = h; };
  /** Sets pen width. */
  void setPenWidth();
  /**
    * Sets properties according to transformation matrix.
    * @param m transformation matrix.
    */
  void setProperties(const QTransform& m);
  /** Returns settings text for item list. */
  QString settings() const;
  /**
    * Sets width.
    * @param w width in cm.
    */
  inline void setWidth( double w) { m_width = w; };
  /** Reimplemented from QGraphicsEllipseItem. */
  virtual int type() const { return Type; };
  /** Applies changes and forces redrawing. */
  void update();
  /** Returns width in cm. */
  inline double width() const { return m_width; };
  /**
   * Writes into plot file.
   * @param plo KConfigGroup object.
   */
  void writePlo(KConfigGroup* plo) const;

protected:
  /** Reimplemented from QGraphicsItem. */
  virtual void mouseReleaseEvent(QGraphicsSceneMouseEvent* event);
  /** Sets properties according to transformation matrix. */
  void setProperties();
  int m_fillStyle;
  double m_width, m_height, m_dir;
};

#endif
