/***************************************************************************
                          residualsdlg.cpp  -  description
                             -------------------
    begin                : Sun Jul 15 2001
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QDesktopWidget>
#include <QPrintDialog>
#include <QPrinter>
#include <QLayout>
#include <KLocalizedString>
#include <kapplication.h>
#include "kplsettings.h"
#include "kpl.h"
#include "residualsdlg.h"
#include "kplscene.h"
#include "kplview.h"
#include "kpldoc.h"

ResidualsDlg::ResidualsDlg(KplDoc* model, KplScene* sc, QWidget* parent) :
 KDialog(parent), scene(sc), m(model)
{
  setCaption(i18n("Residuals"));
  setButtons(Close | User1 | User2);
  setButtonText(User1, i18n("Print"));
  setButtonText(User2, i18n("Save"));
  setButtonFocus(Close);
  QWidget* page = new QWidget;
  setMainWidget(page);
  QVBoxLayout* vbox = new QVBoxLayout(page);
  vbox->setSpacing(spacingHint());
  KplView* view  = new KplView(KplSettings::zoom());
  view->setScene(scene);
  view->setMinimumSize((1.001 * view->sizeHint()).
                       boundedTo(0.8 * QApplication::desktop()->size()));
  vbox->addWidget(view);
}

ResidualsDlg::~ResidualsDlg()
{
}

void ResidualsDlg::slotButtonClicked(int button)
{
  switch (button) {
    case User1: {
      QPrinter printer(QPrinter::HighResolution);
      if (QPrintDialog(&printer).exec() == QDialog::Accepted) {
        QPainter painter(&printer);
        painter.setRenderHint(QPainter::Antialiasing);
        QRectF rs = scene->sceneRect();
        double h = rs.height() * printer.resolution() / logicalDpiY();
        QRect rp = printer.pageRect();
        scene->render(&painter,
                      QRectF(-rp.left(),
                             printer.paperRect().height() - h - rp.top(),
                             rs.width() * printer.resolution() / logicalDpiX(),
                             h), rs, Qt::IgnoreAspectRatio);
      }
      break;
    }
    case User2: {
      KUrl url;
      if (Kpl::getWriteUrl(this, url, KplSettings::currentDir(), "*.plo\n*")) {
        QList<QGraphicsItem*> list = scene->items(Qt::AscendingOrder);
        m->saveDocument(url, list);
      }
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}
