/***************************************************************************
                          splfitdlg.h  -  description
                             -------------------
    begin                : Fri Mar 1 2002
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SPLFITDLG_H
#define SPLFITDLG_H

#include <KDialog>
#include "funitem.h"

class QCheckBox;
class QListWidget;
class ArrayItem;
class DoubleEdit;
class KplDoc;
class SpinBox;
class SplineItem;

/**
  * Spline fit dialog class. Provides a dialog for performing a fit
  * of a smoothing spline to a data set of an array.
  * @author Werner Stille
  */
class SplFitDlg : public KDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param ad pointer to array item.
   * @param fd pointer to spline item.
   * @param mode combination of ShowDlg and AutoFit.
   */
  SplFitDlg(QWidget* parent, KplDoc* model, const ArrayItem* ad,
            const QList<SplineItem*>& fd, int mode);
  /** Destructor. */
  ~SplFitDlg();
  /** Applies settings. */
  void apply();
  /**
   * Performs smoothing spline fit.
   * @param k degree of spline.
   * @param smf smoothing factor.
   * @param xmin lower boundary of approximation interval.
   * @param xmax upper boundary of approximation interval.
   * @param err pointer to structure containing the data error settings.
   * @return error code.
   */
  int fit(int k, double smf, double xmin, double xmax,
          FunItem::DataErrorStruct* err);

protected:
  int dlgMode;
  double xmin, xmax;
  KplDoc* m;
  const ArrayItem* ai;
  SplineItem* sit;
  QList<SplineItem*> items;
  SpinBox* sDeg;
  DoubleEdit *exMin, *exMax, *eFactor;
  QListWidget* results;
  QCheckBox* errCol;
  QPushButton* errMod;
  FunItem::DataErrorStruct err;

protected slots:
  /**
   * Disables/enables error model button.
   * @param state error column state, disables button when true.
   */
  void errColToggled(bool state);
  /** Reimplemented from KDialog. */
  virtual void slotButtonClicked(int button);
  /** Displays the dialog for setting the calculation of error bars. */
  void slotErrMod();
};

#endif
