/***************************************************************************
                          splinedlg.cpp  -  description
                             -------------------
    begin                : Sun Mar 3 2002
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QComboBox>
#include <QFile>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QTextStream>
#include <KColorButton>
#include <KLocalizedString>
#include <KMessageBox>
#include <KUndoStack>
#include "brushdlg.h"
#include "doubleedit.h"
#include "doublespinbox.h"
#include "kpl.h"
#include "kpldoc.h"
#include "kplscene.h"
#include "kplsettings.h"
#include "rootdlg.h"
#include "spinbox.h"
#include "splinedlg.h"
#include "splineitem.h"
#include "symboldlg.h"

SplineDlg::SplineDlg(QWidget* parent, KplDoc* model, SplineItem* si0,
                     QPoint pos) :
 KDialog(parent), m(model), si(si0)
{
  sit = new SplineItem(*si);
  setCaption(i18n("Smoothing spline"));
  setButtons(Help | Ok | Apply | Cancel | User1 | User2);
  setButtonText(User1, i18n("&Export"));
  setButtonText(User2, i18n("R&oots"));
  setButtonFocus(Ok);
  QWidget* page = new QWidget;
  setMainWidget(page);
  QVBoxLayout* vbox = new QVBoxLayout(page);
  vbox->setSpacing(spacingHint());
  QGroupBox* g = new QGroupBox(i18n("Output"));
  QHBoxLayout* hbox = new QHBoxLayout;
  g->setLayout(hbox);
  vbox->addWidget(g);
  hbox->addWidget(eDeriv = new QComboBox);
  QStringList list;
  list << i18n("Integral") << i18n("Spline")
                                 << i18n("1st derivative")
                                 << i18n("2nd derivative")
                                 << i18n("3rd derivative")
                                 << i18n("4th derivative")
                                 << i18n("5th derivative");
  int nDeg = si->degree();
  for (int i = 0; i <= (nDeg + 1); i++)
    eDeriv->addItem(list[i]);
  int nDeriv = si->deriv();
  eDeriv->setCurrentIndex(nDeriv + 1);
  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding));
  hbox->addWidget(new QLabel(i18n("Lower boundary")));
  char frm = KplDoc::presentation();
  int prec = KplSettings::precision();
  hbox->addWidget(exLow = new DoubleEdit(si->xLow(), frm, prec));
  int w = exLow->fontMetrics().width(QString::number(-1.0e-123 / 3.0, frm,
                                                     prec)) + 3;
  exLow->setMinimumWidth(w);
  exLow->setEnabled(nDeriv == -1);
  connect(eDeriv, SIGNAL(activated(int)), SLOT(slotDerivChanged(int)));
  vbox->addWidget(g = new QGroupBox(i18n("Range")));
  QGridLayout* grid = new QGridLayout;
  g->setLayout(grid);
  grid->addWidget(new QLabel("xmin"), 0, 0);
  grid->addWidget(exMin = new DoubleEdit(si->xMin(), frm, prec), 0, 1);
  exMin->setMinimumWidth(w);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid->addWidget(new QLabel("xmax"), 0, 3);
  grid->addWidget(exMax = new DoubleEdit(si->xMax(), frm, prec), 0, 4);
  exMax->setMinimumWidth(w);
  grid->addWidget(new QLabel(QString::fromUtf8("\316\224x")), 1, 0);
  grid->addWidget(exStep = new DoubleEdit(si->deltaX(), frm, prec), 1, 1);
  exStep->setMinimumWidth(w);
  grid->addWidget(new QLabel(i18n("x shift")), 2, 0);
  QPointF step = m->scene()->step();
  grid->addWidget(ex0 = new DoubleSpinBox(si->xOff(), step.x()), 2, 1);
  grid->addWidget(new QLabel(i18n("y shift")), 2, 3);
  grid->addWidget(ey0 = new DoubleSpinBox(si->yOff(), step.y()), 2, 4);
  grid->addWidget(new QLabel(i18n("x normalization")), 3, 0);
  grid->addWidget(efx = new DoubleEdit(si->normX()), 3, 1);
  grid->addWidget(new QLabel(i18n("y normalization")), 3, 3);
  grid->addWidget(efy = new DoubleEdit(si->normY()), 3, 4);
  vbox->addWidget(g = new QGroupBox(i18n("Representation")));
  grid = new QGridLayout;
  g->setLayout(grid);
  grid->addWidget(new QLabel(i18n("Symbol")), 0, 0);
  grid->addWidget(symb = new SpinBox(si->symbol(), -17, 9, 1), 0, 1);
  SymbolButton* bSymb;
  int style = si->fillStyle();
  grid->addWidget(bSymb = new SymbolButton(symb, false, style), 0, 2);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 3);
  grid->addWidget(new QLabel(i18n("Color")), 0, 4);
  grid->addWidget(colData = new KColorButton(si->color()), 0, 5);
  colData->setMinimumWidth(50);
  grid->addWidget(new QLabel(i18n("Size")), 1, 0);
  grid->addWidget(eRelSize = new SpinBox(qRound(100 * si->relSize()), 10, 800,
                                         1), 1, 1);
  eRelSize->setSuffix(" %");
  eRelSize->setDisabled(style);
  grid->addWidget(new QLabel(i18n("Fill pattern")), 1, 4);
  grid->addWidget(fill = new BrushButton(style), 1, 5);
  setHelp("SEC-SPLINE");
  enableButton(User2, si->numK() && (nDeg == 3));
  resize(minimumSize());
  move(pos);
  enableButtonApply(false);
  enableButtonOk(false);
  connect(fill, SIGNAL(valueChanged(bool)), symb, SLOT(setDisabled(bool)));
  connect(fill, SIGNAL(valueChanged(bool)), bSymb, SLOT(setDisabled(bool)));
  connect(fill, SIGNAL(valueChanged(bool)), eRelSize, SLOT(setDisabled(bool)));
  connect(fill, SIGNAL(valueChanged(bool)), SLOT(slotEdited()));
  connect(ex0, SIGNAL(valueChanged(int)), SLOT(slotApply()));
  connect(ey0, SIGNAL(valueChanged(int)), SLOT(slotApply()));
  connect(efx, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(efy, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exStep, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exLow, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(symb, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(eRelSize, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(colData, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
}

SplineDlg::~SplineDlg()
{
  delete sit;
}

void SplineDlg::getTemp()
{
  sit->setDeriv(eDeriv->currentIndex() - 1);
  sit->setXOff(ex0->value());
  sit->setYOff(ey0->value());
  sit->setNormX(efx->value());
  sit->setNormY(efy->value());
  sit->setXMin(exMin->value());
  sit->setXMax(exMax->value());
  sit->setDeltaX(exStep->value());
  sit->setXLow(exLow->value());
  sit->setColor(colData->color().rgb());
  sit->setSymbol(symb->value());
  sit->setRelSize(0.01 * eRelSize->value());
  sit->setFillStyle(fill->value());
}

void SplineDlg::slotApply()
{
  slotButtonClicked(Apply);
}

void SplineDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      getTemp();
      SplineItem* siNew = new SplineItem(*sit);
      m->undoStack()->push(new EditCommand(siNew, si, m));
      si = siNew;
      if (button == Ok)
        accept();
      break;
    }
    case User1: {
      KUrl url;
      if (Kpl::getWriteUrl(this, url, QString(), "*.dat\n*")) {
        QFile f(url.isLocalFile() ? url.path() : KplItem::tmpFile());
        if (f.open(QIODevice::WriteOnly)) {
          getTemp();
          QTextStream ts(&f);
          sit->exportTable(ts, KplDoc::separator(), KplDoc::presentation(),
                           KplSettings::precision());
          f.close();
          m->setCurrentDir(url);
          if (!url.isLocalFile())
            m->copyTmp(f.fileName(), url);
        } else
          KMessageBox::error(this, i18n("while trying to open"));
      }
      break;
    }
    case User2: {
      RootDlg dlg(this, m, si);
      dlg.exec();
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}

void SplineDlg::slotDerivChanged(int index)
{
  exLow->setEnabled(!index);
  slotEdited();
}

void SplineDlg::slotEdited()
{
  enableButtonApply(true);
  enableButtonOk(true);
}
