/***************************************************************************
                          splinedlg.h  -  description
                             -------------------
    begin                : Sun Mar 3 2002
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SPLINEDLG_H
#define SPLINEDLG_H

#include <KDialog>

class QComboBox;
class KColorButton;
class BrushButton;
class DoubleEdit;
class DoubleSpinBox;
class KplDoc;
class SpinBox;
class SplineItem;

/**
  * Spline settings dialog class. Provides a dialog for the settings
  * of a smoothing spline.
  * @author Werner Stille
  */
class SplineDlg : public KDialog
{
  Q_OBJECT
public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param fd pointer to spline object.
   * @param pos dialog position.
   */
  SplineDlg(QWidget* parent, KplDoc* model, SplineItem* fd, QPoint pos);
  /** Destructor. */
  ~SplineDlg();

protected:
  /** Copies settings to temporary spline object. */
  void getTemp();
  KplDoc* m;
  SplineItem *si, *sit;
  QComboBox *eDeriv;
  BrushButton *fill;
  DoubleEdit *exLow, *exMin, *exMax, *exStep, *efx, *efy;
  DoubleSpinBox *ex0, *ey0;
  SpinBox *symb, *eRelSize;
  KColorButton* colData;

protected slots:
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /** Reimplemented from KDialog. */
  virtual void slotButtonClicked(int button);
  /**
   * Enables lower boundary widget for integrals.
   * @param index index of selected derivative box item.
   */
  void slotDerivChanged(int index);
  /** Enables buttons. */
  void slotEdited();
};

#endif
