/***************************************************************************
                          symboldlg.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 2011 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QButtonGroup>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QRadioButton>
#include <QSpinBox>
#include <KLocalizedString>
#include "kplitem.h"
#include "symboldlg.h"

SymbolButton::SymbolButton(QSpinBox* sb, bool lines, bool disable,
                           QWidget* parent) :
 QPushButton(parent), symb(sb), linesOnly(lines)
{
  setIconSize(QSize(50, 16));
  setFixedWidth(80);
  setButtonSymbol(symb->value());
  if (disable) {
    setDisabled(disable);
    symb->setDisabled(disable);
  }
  connect(symb, SIGNAL(valueChanged(int)), SLOT(setButtonSymbol(int)));
  connect(this, SIGNAL(clicked()), SLOT(slotSymbol()));
}

SymbolButton::~SymbolButton()
{
}

void SymbolButton::setButtonSymbol(int isymb)
{
  setIcon((isymb == -17) ? QIcon() : KplItem::pixmap(isymb));
}

void SymbolButton::slotSymbol()
{
  SymbolDlg dlg(parentWidget(), symb, linesOnly);
  dlg.exec();
}

SymbolDlg::SymbolDlg(QWidget* parent, QSpinBox* sb0, bool lines) :
 KDialog(parent), sb(sb0), linesOnly(lines)
{
  setCaption(i18n(lines ? "Lines" : "Lines and markers"));
  setButtons(Ok | Cancel);
  QWidget* page = new QWidget;
  setMainWidget(page);
  QVBoxLayout* vbox = new QVBoxLayout(page);
  vbox->setSpacing(spacingHint());
  QGroupBox* g1 = new QGroupBox(linesOnly ? QString("") : i18n("Lines"));
  bg = new QButtonGroup(g1);
  QRadioButton* r;
  QLabel* l;
  QGridLayout* grid;
  QHBoxLayout* hbox;
  if (linesOnly)
    vbox->addWidget(g1);
  else {
    vbox->addLayout(hbox = new QHBoxLayout);
    hbox->addWidget(g1);
    QGroupBox* g2 = new QGroupBox(i18n("Markers"));
    hbox->addWidget(g2);
    g2->setLayout(grid = new QGridLayout);
    for (int i = 17; i >= 0; i--) {
      int ix = 3 * (i / 9);
      int iy = i % 9;
      grid->addWidget(r = new QRadioButton(QString::number(-i)), iy, ix + 1);
      bg->addButton(r, -i - 2);
      if (i == 17)
        continue;
      grid->addWidget(l = new QLabel, iy, ix);
      l->setPixmap(KplItem::pixmap(-i));
    }
    grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding),
                  0, 2);
  }
  g1->setLayout(grid = new QGridLayout);
  for (int i = 1; i < 10; i++) {
    grid->addWidget(r = new QRadioButton(QString::number(i)), i - 1, 1);
    bg->addButton(r, i);
    grid->addWidget(l = new QLabel, i - 1, 0);
    l->setPixmap(KplItem::pixmap(i));
  }
  int i = sb->value();
  bg->button((i < 1) ? i - 2 : i)->setChecked(true);
}

SymbolDlg::~SymbolDlg()
{
}

void SymbolDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Ok: {
      int i = bg->checkedId();
      sb->setValue((i < 0) ? i + 2 : i);
      accept();
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}
