//
// C++ Implementation: threeddlg
//
// Description: 
//
//
// Author: Werner Stille <stille@uni-freiburg.de>, (C) 2011
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include <QCheckBox>
#include <QDoubleSpinBox>
#include <QFile>
#include <QGroupBox>
#include <QLabel>
#include <QLayout>
#include <QPushButton>
#include <QRadioButton>
#include <QToolButton>
#include <KButtonGroup>
#include <KColorButton>
#include <KLocalizedString>
#include <KMessageBox>
#include "arrayitem.h"
#include "doubleedit.h"
#include "editdlg.h"
#include "editparameterdlg.h"
#include "edittextdlg.h"
#include "funcdlg.h"
#include "kpl.h"
#include "kpldoc.h"
#include "kplsettings.h"
#include "spinbox.h"
#include "threeddlg.h"
#include "threeditem.h"

ThreeDDlg::ThreeDDlg(QWidget* parent, KplDoc* model, ThreeDItem* td0,
                     QPoint pos, const QString& caption, const QString anchor,
                     const QString& name1) :
 KPageDialog(parent), m(model), td(td0)
{
  setFaceType(KPageDialog::Tabbed);
  setCaption(caption);
  setButtons(Help | Ok | Apply | Cancel | User1);
  setButtonText(User1, i18n("&Export"));
  setHelp(anchor);
  page1 = new QWidget;
  vbox1 = new QVBoxLayout(page1);
  vbox1->setSpacing(spacingHint());
  g1 = new QGroupBox(i18n("Range"));
  addPage(page1, name1);
  QWidget* page;
  QGridLayout* grid = new QGridLayout(page = new QWidget);
  grid->setSpacing(spacingHint());
  QGroupBox* g;
  grid->addWidget(g = new QGroupBox(i18n("Window")), 0, 0, 1, 2);
  QGridLayout* grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel(i18n("Left margin")), 0, 0);
  grid2->addWidget(ex0 = new QDoubleSpinBox, 0, 1);
  ex0->setMaximum(200);
  ex0->setSingleStep(0.1);
  ex0->setSuffix(" cm");
  ex0->setValue(td->x0());
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid2->addWidget(new QLabel(i18n("Width")), 0, 3);
  grid2->addWidget(exl = new QDoubleSpinBox, 0, 4);
  exl->setRange(0.5, 200);
  exl->setSingleStep(0.1);
  exl->setSuffix(" cm");
  exl->setValue(td->width());
  grid2->addWidget(new QLabel(i18n("Bottom margin")), 1, 0);
  grid2->addWidget(ey0 = new QDoubleSpinBox, 1, 1);
  ey0->setMaximum(200);
  ey0->setSingleStep(0.1);
  ey0->setSuffix(" cm");
  ey0->setValue(td->y0());
  grid2->addWidget(new QLabel(i18n("Height")), 1, 3);
  grid2->addWidget(eyl = new QDoubleSpinBox, 1, 4);
  eyl->setRange(0.5, 200);
  eyl->setSingleStep(0.1);
  eyl->setSuffix(" cm");
  eyl->setValue(td->height());
  grid->addWidget(g = new QGroupBox(i18n("Reference point")), 0, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("x"), 0, 0);
  grid2->addWidget(eXref = new QDoubleSpinBox, 0, 1);
  eXref->setMaximum(200);
  eXref->setSingleStep(0.1);
  eXref->setSuffix(" cm");
  eXref->setValue(td->xRef());
  grid2->addWidget(new QLabel("y"), 1, 0);
  grid2->addWidget(eYref = new QDoubleSpinBox, 1, 1);
  eYref->setMaximum(200);
  eYref->setSingleStep(0.1);
  eYref->setSuffix(" cm");
  eYref->setValue(td->yRef());
  grid->addWidget(g = new QGroupBox(i18n("3D box")), 1, 0);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("x"), 0, 0);
  grid2->addWidget(eBoxx = new QDoubleSpinBox, 0, 1);
  eBoxx->setRange(0.5, 200);
  eBoxx->setSingleStep(0.1);
  eBoxx->setSuffix(" cm");
  eBoxx->setValue(td->boxX());
  grid2->addWidget(new QLabel("y"), 1, 0);
  grid2->addWidget(eBoxy = new QDoubleSpinBox, 1, 1);
  eBoxy->setRange(0.5, 200);
  eBoxy->setSingleStep(0.1);
  eBoxy->setSuffix(" cm");
  eBoxy->setValue(td->boxY());
  grid2->addWidget(new QLabel("z"), 2, 0);
  grid2->addWidget(eBoxz = new QDoubleSpinBox, 2, 1);
  eBoxz->setRange(0.5, 200);
  eBoxz->setSingleStep(0.1);
  eBoxz->setSuffix(" cm");
  eBoxz->setValue(td->boxZ());
  grid->addWidget(g = new QGroupBox(i18n("Rotation")), 1, 1);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("phi"), 0, 0);
  grid2->addWidget(ePhi = new QDoubleSpinBox, 0, 1);
  ePhi->setRange(-360, 360);
  ePhi->setSuffix("\260");
  ePhi->setValue(td->phi());
  grid2->addWidget(new QLabel("theta"), 1, 0);
  grid2->addWidget(eTheta = new QDoubleSpinBox, 1, 1);
  eTheta->setRange(-360, 360);
  eTheta->setSuffix("\260");
  eTheta->setValue(td->theta());
  grid->addWidget(g = new QGroupBox(i18n("Automatic")), 1, 2);
  QVBoxLayout* vbox2 = new QVBoxLayout(g);
  vbox2->addWidget(autoNorm = new QCheckBox(i18n("Normalization")));
  autoNorm->setChecked(KplSettings::normalization() &&
                       (!(td->logX() || td->logY() || td->logZ())));
  bAuto = new QPushButton(i18n("Autoscale"));
  vbox2->addWidget(bAuto);
  grid->addWidget(g = new QGroupBox(i18n("Axes")), 2, 0, 1, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("xtic"), 0, 0);
  grid2->addWidget(exTic = new DoubleEdit(td->xTic()), 0, 1);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid2->addWidget(new QLabel("mticx"), 0, 3);
  grid2->addWidget(emTicx = new SpinBox(td->minTicX(), 0, 10), 0, 4);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 5);
  grid2->addWidget(new QLabel("ndigx"), 0, 6);
  grid2->addWidget(enDigx = new SpinBox(td->numDigX(), -1, 8), 0, 7);
  enDigx->setEnabled(!td->logX());
  grid2->addWidget(new QLabel("ytic"), 1, 0);
  grid2->addWidget(eyTic = new DoubleEdit(td->yTic()), 1, 1);
  grid2->addWidget(new QLabel("mticy"), 1, 3);
  grid2->addWidget(emTicy = new SpinBox(td->minTicY(), 0, 10), 1, 4);
  grid2->addWidget(new QLabel("ndigy"), 1, 6);
  grid2->addWidget(enDigy = new SpinBox(td->numDigY(), -1, 8), 1, 7);
  enDigy->setEnabled(!td->logY());
  grid2->addWidget(new QLabel("ztic"), 2, 0);
  grid2->addWidget(ezTic = new DoubleEdit(td->zTic()), 2, 1);
  grid2->addWidget(new QLabel("mticz"), 2, 3);
  grid2->addWidget(emTicz = new SpinBox(td->minTicZ(), 0, 10, 1), 2, 4);
  grid2->addWidget(new QLabel("ndigz"), 2, 6);
  grid2->addWidget(enDigz = new SpinBox(td->numDigZ(), -1, 8), 2, 7);
  enDigz->setEnabled(!td->logZ());
  grid->addWidget(g = new QGroupBox(i18n("Normalization")), 2, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("x"), 0, 0);
  grid2->addWidget(efx = new DoubleEdit(td->fx()), 0, 1);
  grid2->addWidget(new QLabel("y"), 1, 0);
  grid2->addWidget(efy = new DoubleEdit(td->fy()), 1, 1);
  grid2->addWidget(new QLabel("z"), 2, 0);
  grid2->addWidget(efz = new DoubleEdit(td->fz()), 2, 1);
  addPage(page, i18n("Scaling"));
  grid = new QGridLayout(page = new QWidget);
  grid->setSpacing(spacingHint());
  grid->addWidget(bg1 = new KButtonGroup, 0, 0);
  bg1->setTitle(i18n("Axis mode"));
  QVBoxLayout* vbox = new QVBoxLayout(bg1);
  vbox->addWidget(new QRadioButton(i18n("No Tics")));
  vbox->addWidget(new QRadioButton(i18n("Tics")));
  vbox->addWidget(new QRadioButton(i18n("Tics with numbers")));
  bg1->setSelected(td->gridMode());
  grid->addWidget(bg2 = new KButtonGroup, 0, 1);
  bg2->setTitle(i18n("3D mode"));
  vbox = new QVBoxLayout(bg2);
  vbox->addWidget(new QRadioButton("x"));
  vbox->addWidget(new QRadioButton("z"));
  vbox->addWidget(new QRadioButton(i18n("x and z")));
  bg2->setSelected(td->mode3D());
  grid->addWidget(g = new QGroupBox(i18n("Colors")), 1, 0);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel(i18n("Frame")), 0, 0);
  grid2->addWidget(colFrame = new KColorButton(QColor(td->colFrame())), 0, 1);
  grid2->addWidget(new QLabel(i18n("Tics")), 1, 0);
  grid2->addWidget(colGrid = new KColorButton(QColor(td->colGrid())), 1, 1);
  grid2->addWidget(new QLabel(i18n("Data")), 2, 0);
  grid2->addWidget(colData = new KColorButton(QColor(td->colData())), 2, 1);
  grid->addWidget(g = new QGroupBox(i18n("General")), 1, 1);
  vbox = new QVBoxLayout(g);
  vbox->addWidget(modeFrame = new QCheckBox(i18n("Show frame")));
  modeFrame->setChecked(td->showFrame());
  QHBoxLayout* hbox = new QHBoxLayout;
  vbox->addLayout(hbox);
  hbox->addWidget(new QLabel(i18n("Line width")));
  hbox->addWidget(eRelSiz = new SpinBox(100 * td->relSize(), 10, 800));
  eRelSiz->setSuffix(" %");
  grid->addWidget(g = new QGroupBox(i18n("Labels")), 2, 0, 1, 2);
  grid2 = new QGridLayout(g);
  grid2->addWidget(new QLabel("iex"), 0, 0);
  grid2->addWidget(ex = new SpinBox(td->expX(), -300, 300), 0, 1);
  grid2->addItem(new QSpacerItem(20, 10, QSizePolicy::Minimum), 0, 2);
  grid2->addWidget(new QLabel(i18n("x axis")), 0, 3);
  grid2->addWidget(eLetX = new QLineEdit(td->textX()), 0, 4);
  QToolButton* tb;
  grid2->addWidget(tb = new QToolButton, 0, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetX()));
  grid2->addWidget(new QLabel("iey"), 1, 0);
  grid2->addWidget(ey = new SpinBox(td->expY(), -300, 300), 1, 1);
  grid2->addWidget(new QLabel(i18n("y axis")), 1, 3);
  grid2->addWidget(eLetY = new QLineEdit(td->textY()), 1, 4);
  grid2->addWidget(tb = new QToolButton, 1, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetY()));
  grid2->addWidget(new QLabel("iez"), 2, 0);
  grid2->addWidget(ez = new SpinBox(td->expZ(), -300, 300), 2, 1);
  grid2->addWidget(new QLabel(i18n("z axis")), 2, 3);
  grid2->addWidget(eLetZ = new QLineEdit(td->textZ()), 2, 4);
  grid2->addWidget(tb = new QToolButton, 2, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetZ()));
  grid2->addWidget(new QLabel(i18n("Heading")), 3, 3);
  grid2->addWidget(eLetH = new QLineEdit(td->textH()), 3, 4);
  grid2->addWidget(tb = new QToolButton, 3, 5);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotEditLetH()));
  connect(autoNorm, SIGNAL(toggled(bool)), SLOT(slotNormToggled(bool)));
  addPage(page, i18n("Representation"));
  move(pos);
  enableButtonApply(false);
  enableButtonOk(false);
}

ThreeDDlg::~ThreeDDlg()
{
}

void ThreeDDlg::autoScale(double xmin, double xmax, double ymin, double ymax,
                          double zmin, double zmax)
{
  double w = exl->value();
  double h = eyl->value();
  double r = h / w;
  double phi = VectorItem::deg2rad * ePhi->value();
  double theta = VectorItem::deg2rad * eTheta->value();
  double ctheta = cos(theta);
  double xr = fabs(sin(phi)) + fabs(cos(phi));
  double b = 0.7 * qMin(w / xr, h / (r * fabs(ctheta) +
                                     xr * fabs(sin(theta))));
  eBoxx->setValue(b);
  eBoxz->setValue(b);
  eBoxy->setValue(b * r);
  eXref->setValue(0.5 * w);
  eYref->setValue(0.5 * (h - b * r * ctheta));
  int iex, iey, iez;
  double fx, fy, fz;
  if (autoNorm->isChecked()) {
    ScaledItem::exponent(qMax(fabs(xmin), fabs(xmax)), &iex, &fx);
    ScaledItem::exponent(qMax(fabs(ymin), fabs(ymax)), &iey, &fy);
    ScaledItem::exponent(qMax(fabs(zmin), fabs(zmax)), &iez, &fz);
  } else {
    iex = iey = iez = 0;
    fx = fy = fz = 1.0;
  }
  efx->setValue(fx);
  ex->setValue(iex);
  efy->setValue(fy);
  ey->setValue(iey);
  efz->setValue(fz);
  ez->setValue(iez);
  int mtic, ndig;
  double vmin, vman, vtic;
  FrameItem::autoScale(&vmin, &vman, &vtic, &mtic, &ndig, xmin, xmax, fx, 0.25,
                       logX->isChecked());
  exMin->setValue(vmin / fx);
  exMax->setValue(vman / fx);
  exTic->setValue(vtic);
  emTicx->setValue(mtic);
  enDigx->setValue(ndig);
  FrameItem::autoScale(&vmin, &vman, &vtic, &mtic, &ndig, ymin, ymax, fy, 0.25,
                       logY->isChecked());
  eyMin->setValue(vmin / fy);
  eyMax->setValue(vman / fy);
  eyTic->setValue(vtic);
  emTicy->setValue(mtic);
  enDigy->setValue(ndig);
  FrameItem::autoScale(&vmin, &vman, &vtic, &mtic, &ndig, zmin, zmax, fz, 0.25,
                       logZ->isChecked());
  ezMin->setValue(vmin / fz);
  ezMax->setValue(vman / fz);
  ezTic->setValue(vtic);
  emTicz->setValue(mtic);
  enDigz->setValue(ndig);
}

void ThreeDDlg::getValues(ThreeDItem* ti)
{
  ti->setXMin(exMin->value());
  ti->setXMax(exMax->value());
  ti->setDeltaX(exStep->value());
  ti->setLogX(logX->isChecked());
  ti->setYMin(eyMin->value());
  ti->setYMax(eyMax->value());
  ti->setLogY(logY->isChecked());
  ti->setZMin(ezMin->value());
  ti->setZMax(ezMax->value());
  ti->setDeltaZ(ezStep->value());
  ti->setLogZ(logZ->isChecked());
  ti->setX0(ex0->value());
  ti->setWidth(exl->value());
  ti->setY0(ey0->value());
  ti->setHeight(eyl->value());
  ti->setFx(efx->value());
  ti->setFy(efy->value());
  ti->setFz(efz->value());
  ti->setBoxX(eBoxx->value());
  ti->setBoxY(eBoxy->value());
  ti->setBoxZ(eBoxz->value());
  ti->setXTic(exTic->value());
  ti->setYTic(eyTic->value());
  ti->setZTic(ezTic->value());
  ti->setMinTicX(emTicx->value());
  ti->setMinTicY(emTicy->value());
  ti->setMinTicZ(emTicz->value());
  ti->setNumDigX(enDigx->value());
  ti->setNumDigY(enDigy->value());
  ti->setNumDigZ(enDigz->value());
  ti->setPhi(ePhi->value());
  ti->setTheta(eTheta->value());
  ti->setXRef(eXref->value());
  ti->setYRef(eYref->value());
  ti->setGridMode(bg1->selected());
  ti->setMode3D(bg2->selected());
  ti->setShowFrame(modeFrame->isChecked());
  ti->setRelSize(0.01 * eRelSiz->value());
  ti->setColFrame(colFrame->color().rgb());
  ti->setColGrid(colGrid->color().rgb());
  ti->setColData(colData->color().rgb());
  ti->setExpX(ex->value());
  ti->setExpY(ey->value());
  ti->setExpZ(ez->value());
  ti->setTextX(eLetX->text());
  ti->setTextY(eLetY->text());
  ti->setTextZ(eLetZ->text());
  ti->setTextH(eLetH->text());
}

void ThreeDDlg::initRange(QGridLayout* grid, int row)
{
  char format = KplDoc::presentation();
  int prec = KplSettings::precision();
  grid->addWidget(new QLabel("xmin"), row, 0);
  grid->addWidget(exMin = new DoubleEdit(td->xMin(), format, prec), row, 1);
  int w = exMin->fontMetrics().width(QString::number(-1.0e-123 / 3.0, format,
                                                     prec)) + 3;
  exMin->setMinimumWidth(w);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row,
                                2);
  grid->addWidget(new QLabel("xmax"), row, 3);
  grid->addWidget(exMax = new DoubleEdit(td->xMax(), format, prec), row, 4);
  exMax->setMinimumWidth(w);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row,
                                5);
  grid->addWidget(new QLabel(QString::fromUtf8("\316\224x")), row, 6);
  grid->addWidget(exStep = new DoubleEdit(td->deltaX(), format, prec), row, 7);
  exStep->setMinimumWidth(w);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row,
                                8);
  grid->addWidget(logX = new QCheckBox("log"), row++, 9);
  logX->setChecked(td->logX());
  grid->addWidget(new QLabel("ymin"), row, 0);
  grid->addWidget(eyMin = new DoubleEdit(td->yMin()), row, 1);
  grid->addWidget(new QLabel("ymax"), row, 3);
  grid->addWidget(eyMax = new DoubleEdit(td->yMax()), row, 4);
  grid->addWidget(logY = new QCheckBox("log"), row++, 9);
  logY->setChecked(td->logY());
  grid->addWidget(new QLabel("zmin"), row, 0);
  grid->addWidget(ezMin = new DoubleEdit(td->zMin(), format, prec), row, 1);
  ezMin->setMinimumWidth(w);
  grid->addWidget(new QLabel("zmax"), row, 3);
  grid->addWidget(ezMax = new DoubleEdit(td->zMax(), format, prec), row, 4);
  ezMax->setMinimumWidth(w);
  grid->addWidget(new QLabel(QString::fromUtf8("\316\224z")), row, 6);
  grid->addWidget(ezStep = new DoubleEdit(td->deltaZ(), format, prec), row, 7);
  ezStep->setMinimumWidth(w);
  grid->addWidget(logZ = new QCheckBox("log"), row, 9);
  logZ->setChecked(td->logZ());
  connect(bg1, SIGNAL(changed(int)), SLOT(slotEdited()));
  connect(bg2, SIGNAL(changed(int)), SLOT(slotEdited()));
  connect(logX, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(logY, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(logZ, SIGNAL(toggled(bool)), SLOT(slotLogToggled(bool)));
  connect(bAuto, SIGNAL(clicked()), SLOT(slotAutoScale()));
  connect(ex0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(exl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(ey0, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eyl, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(exMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exStep, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezMin, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezMax, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezStep, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(efx, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(efy, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(efz, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(exTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(eyTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ezTic, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(ePhi, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eTheta, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eXref, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eYref, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eBoxx, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eBoxy, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(eBoxz, SIGNAL(valueChanged(const QString&)), SLOT(slotEdited()));
  connect(emTicx, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(emTicy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(emTicz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigx, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(enDigz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ex, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ey, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ez, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(eRelSiz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(colFrame, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(colGrid, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(colData, SIGNAL(changed(const QColor&)), SLOT(slotEdited()));
  connect(modeFrame, SIGNAL(toggled(bool)), SLOT(slotEdited()));
  connect(eLetX, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetY, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetZ, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(eLetH, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
}

void ThreeDDlg::slotEdited()
{
  enableButtonApply(true);
  enableButtonOk(true);
}

void ThreeDDlg::slotEditLetX()
{
  EditTextDlg::editText(this, eLetX);
}

void ThreeDDlg::slotEditLetY()
{
  EditTextDlg::editText(this, eLetY);
}

void ThreeDDlg::slotEditLetZ()
{
  EditTextDlg::editText(this, eLetZ);
}

void ThreeDDlg::slotEditLetH()
{
  EditTextDlg::editText(this, eLetH);
}

void ThreeDDlg::slotLogToggled(bool state)
{
  const QCheckBox* b = (const QCheckBox*) sender();
  if ((b == logX) || (b == logY) || (b == logZ)) {
    SpinBox* nd;
    DoubleEdit *mi, *ma;
    if (b == logX) {
      nd = enDigx;
      mi = exMin;
      ma = exMax;
    } else
      if (b == logY) {
        nd = enDigy;
        mi = eyMin;
        ma = eyMax;
      } else {
        nd = enDigz;
        mi = ezMin;
        ma = ezMax;
      }
    nd->setEnabled(!state);
    double bottom = state ? 1.0e-300 : -1.0e300;
    mi->setBottom(bottom);
    ma->setBottom(bottom);
  }
  if (state)
    autoNorm->setChecked(false);
  slotEdited();
}

void ThreeDDlg::slotNormToggled(bool state)
{
  if (state) {
    logX->setChecked(false);
    logY->setChecked(false);
    logZ->setChecked(false);
  }
}

Fun3DDlg::Fun3DDlg(QWidget* parent, KplDoc* model, Fun3DItem *fi0,
                   QPoint pos) :
 ThreeDDlg(parent, model, fi0, pos, i18n("3D function"), "SEC-FUNC3D",
           i18n("Function")), fi(fi0), fit(new Fun3DItem(*fi0))
{
  QGroupBox* g = new QGroupBox("y(x,z)");
  vbox1->addWidget(g);
  QGridLayout* grid = new QGridLayout(g);
  grid->addWidget(new QLabel(i18n("Library")), 0, 0);
  QHBoxLayout* hbox = new QHBoxLayout;
  grid->addLayout(hbox, 0, 1);
  hbox->addWidget(fileName = new QLineEdit(fi->urlY().isLocalFile() ?
                                           fi->urlY().path() :
                                           fi->urlY().url()));
  QToolButton* tb = new QToolButton;
  tb->setText("...");
  hbox->addWidget(tb);
  connect(tb, SIGNAL(clicked()), SLOT(slotFile()));
  grid->addWidget(new QLabel(i18n("Function")), 1, 0);
  grid->addLayout(hbox = new QHBoxLayout, 1, 1);
  hbox->addWidget(func = new QLineEdit(fi->nameY()));
  hbox->addWidget(tb = new QToolButton);
  tb->setText("...");
  connect(tb, SIGNAL(clicked()), SLOT(slotFunc()));
  grid->addWidget(new QLabel(i18n("Parameter")), 2, 0);
  grid->addLayout(hbox = new QHBoxLayout, 2, 1);
  QPushButton* b;
  hbox->addWidget(b = new QPushButton(i18n("Edit")));
  connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Load")));
  connect(b, SIGNAL(clicked()), SLOT(slotGetParameter()));
  hbox->addWidget(b = new QPushButton(i18n("Save")));
  connect(b, SIGNAL(clicked()), SLOT(slotSaveParameter()));
  vbox1->addWidget(g1);
  initRange(new QGridLayout(g1));
  connect(fileName, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  connect(func, SIGNAL(textChanged(const QString&)), SLOT(slotEdited()));
  restoreDialogSize(KGlobal::config()->group("Func3DDialog"));
}

Fun3DDlg::~Fun3DDlg()
{
  KConfigGroup config = KGlobal::config()->group("Func3DDialog");
  saveDialogSize(config);
  delete fit;
}

bool Fun3DDlg::getTemp()
{
  if (fit->setFuncY(fileName->text(), func->text())) {
    fit->setXMin(exMin->value());
    fit->setXMax(exMax->value());
    fit->setZMin(ezMin->value());
    fit->setZMax(ezMax->value());
    return true;
  } else
    return false;
}

void Fun3DDlg::slotAutoScale()
{
  if (getTemp()) {
    int nv = fit->calcTable();
    double ymi, yma;
    ScaledItem::limits(&ymi, &yma, fit->yV()[0], nv);
    autoScale(fit->xMin(), fit->xMax(), ymi, yma, fit->zMin(), fit->zMax());
  }
}

void Fun3DDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      if (fit->setFuncY(fileName->text(), func->text())) {
        Fun3DItem* fiNew = new Fun3DItem(*fit);
        getValues(fiNew);
        m->undoStack()->push(new EditCommand(fiNew, fi, m));
        fi = fiNew;
        if (button == Ok)
          accept();
      }
      break;
    }
    case User1:
      if (getTemp()) {
        KUrl url;
        if (Kpl::getWriteUrl(this, url, QString(), "*.dat\n*")) {
          QFile f(url.isLocalFile() ? url.path() : KplItem::tmpFile());
          if (f.open(QIODevice::WriteOnly)) {
            QTextStream ts(&f);
            fit->exportTable(ts, KplDoc::separator(), KplDoc::presentation(),
                             KplSettings::precision());
            f.close();
            m->setCurrentDir(url);
            if (!url.isLocalFile())
              m->copyTmp(f.fileName(), url);
          } else
            KMessageBox::error(this, i18n("while trying to open"));
        }
      }
      break;
    default:
      KDialog::slotButtonClicked(button);
  }
}

void Fun3DDlg::slotEditParameter()
{
  EditParameterDlg dlg(this, fit->pY(), fileName->text(), func->text(), true);
  connect(&dlg, SIGNAL(applyClicked()), SLOT(slotApply()));
  dlg.exec();
}

void Fun3DDlg::slotFile()
{
  m->getFile(fileName);
}

void Fun3DDlg::slotFunc()
{
  FuncDlg::getFunc(this, fileName, func);
}

void Fun3DDlg::slotGetParameter()
{
  m->getPar(this, fit->pY());
}

void Fun3DDlg::slotSaveParameter()
{
  m->saveFunPar(this, fit->pY());
}

Array3DDlg::Array3DDlg(QWidget* parent, KplDoc* model, Array3DItem *ai0,
                       QPoint pos) :
 ThreeDDlg(parent, model, ai0, pos, i18n("3D array"), "SEC-ARRAY3D",
           i18n("Array")), ai(ai0), ait(new Array3DItem(*ai0))
{
  QGroupBox* g = new QGroupBox(i18n("Data"));
  vbox1->addWidget(g);
  QGridLayout* grid = new QGridLayout(g);
  grid->addWidget(new QLabel(i18n("File")), 0, 0);
  QHBoxLayout* hbox;
  grid->addLayout(hbox = new QHBoxLayout, 0, 1, 1, 3);
  hbox->addWidget(fileName = new QLabel);
  bool b_int = ai->internal();
  if (!b_int) {
    const KUrl& url = ai->url();
    fileName->setText(url.isLocalFile() ? url.path() : url.url());
  }
  fileName->setFrameStyle(QFrame::Panel | QFrame::Sunken);
  fileName->setSizePolicy(QSizePolicy::MinimumExpanding,
                          QSizePolicy::Minimum);
  QToolButton* tb = new QToolButton;
  tb->setText("...");
  hbox->addWidget(tb);
  connect(tb, SIGNAL(clicked()), SLOT(slotFile()));
  hbox->addWidget(bReload = new QToolButton);
  bReload->setIcon(KIcon("view-refresh"));
  bReload->setDisabled(fileName->text().isEmpty());
  connect(bReload, SIGNAL(clicked()), SLOT(slotReload()));
  grid->addLayout(hbox = new QHBoxLayout, 1, 1);
  hbox->addWidget(internal = new QCheckBox(i18n("Internal data")));
  internal->setChecked(ai->internal());
  hbox->addWidget(edit = new QPushButton(i18n("&Edit")));
  edit->setEnabled(ai->internal());
  connect(edit, SIGNAL(clicked()), SLOT(slotEdit()));
  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding));
  vbox1->addWidget(g = new QGroupBox(i18n("Columns")));
  grid = new QGridLayout(g);
  grid->addWidget(new QLabel(i18n("x column")), 0, 0);
  grid->addWidget(ix = new SpinBox, 0, 1);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 2);
  grid->addWidget(new QLabel(i18n("y column")), 0, 3);
  grid->addWidget(iy = new SpinBox, 0, 4);
  grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), 0, 5);
  grid->addWidget(new QLabel(i18n("z column")), 0, 6);
  grid->addWidget(iz = new SpinBox, 0, 7);
  grid->addWidget(new QLabel(i18n("Error column")), 1, 0);
  grid->addWidget(ie = new SpinBox, 1, 1);
  ie->setEnabled(ai->errBars());
  grid->addWidget(err = new QCheckBox(i18n("Error bar")), 1, 2, 1, 4);
  err->setChecked(ai->errBars());
  grid->addWidget(new QLabel(i18n("Smoothing factor")), 2, 0);
  grid->addWidget(eFactor = new DoubleEdit(ai->smf()), 2, 1);
  vbox1->addWidget(g1);
  grid = new QGridLayout(g1);
  grid->addWidget(new QLabel(i18n("Start index")), 0, 0);
  grid->addWidget(iStart = new SpinBox(0, 0, 1), 0, 1);
  grid->addWidget(new QLabel(i18n("Number of points")), 0, 3);
  grid->addWidget(n = new SpinBox(0, 0, 1), 0, 4);
  setSpinBoxes(ai);
  initRange(grid, 1);
  connect(ix, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(iy, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(iz, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(ie, SIGNAL(valueChanged(int)), SLOT(slotEdited()));
  connect(internal, SIGNAL(clicked(bool)), SLOT(slotInternal(bool)));
  connect(err, SIGNAL(toggled(bool)), ie, SLOT(setEnabled(bool)));
  connect(err, SIGNAL(clicked(bool)), SLOT(slotEdited()));
  connect(eFactor, SIGNAL(textEdited(const QString&)), SLOT(slotEdited()));
  connect(iStart, SIGNAL(valueChanged(int)), SLOT(slotStartChanged(int)));
  connect(n, SIGNAL(valueChanged(int)), SLOT(slotNChanged(int)));
  restoreDialogSize(KGlobal::config()->group("Array3DDialog"));
}

Array3DDlg::~Array3DDlg()
{
  KConfigGroup config = KGlobal::config()->group("Array3DDialog");
  saveDialogSize(config);
  delete ait;
}

bool Array3DDlg::load(KUrl &url, int idec)
{
  bool success = !url.isEmpty();
  if (success) {
    ait->setUrl(url);
    ait->setIDec(idec);
    success = readData();
  }
  return success;
}

bool Array3DDlg::readData(bool fromBuffer)
{
  int ncols;
  ait->setNumRows(fromBuffer ?
                  ArrayItem::readFile(&buf, ncols, ait->data()) :
                  ArrayItem::readFile(ait->url(), ncols, ait->data(),
                                      ait->iDec()));
  if (ait->numRows()) {
    ait->setNumCols(ncols);
    int maxCol = ncols - 1;
    ait->setColX(qMin(ai->colX(), maxCol));
    ait->setColY(qMin(ai->colY(), maxCol));
    ait->setColZ(qMin(ai->colZ(), maxCol));
    ait->setColE(qMin(ai->colE(), maxCol));
    ait->setRowStart(qMin(ai->rowStart(), ait->numRows() - 1));
    ait->setCount(ait->numRows() - ait->rowStart());
    if (ai->count() && (!fromBuffer))
      ait->setCount(qMin(ai->count(), ait->count()));
    return true;
  } else {
    KMessageBox::error(this, i18n("while reading the data"));
    return false;
  }
}

void Array3DDlg::setSpinBoxes(Array3DItem* ad)
{
  int iColMax = qMax(0, ad->numCols() - 1);
  ix->setRange(0, iColMax);
  ix->setValue(ad->colX());
  iy->setRange(0, iColMax);
  iy->setValue(ad->colY());
  iz->setRange(0, iColMax);
  iz->setValue(ad->colZ());
  ie->setRange(0, iColMax);
  ie->setValue(ad->colE());
  iStart->setRange(0, qMax(0, ad->numRows() - 1));
  iStart->setValue(ad->rowStart());
  n->setRange(0, ad->numRows());
  n->setValue(ad->count());
}

void Array3DDlg::slotAutoScale()
{
  if (*ait->data()) {
    double xmi, xma, ymi, yma, zmi, zma;
    int istart = iStart->value();
    int nv = n->value();
    ScaledItem::limits(&xmi, &xma, &(*ait->data())[ix->value()][istart], nv);
    ScaledItem::limits(&ymi, &yma, &(*ait->data())[iy->value()][istart], nv);
    ScaledItem::limits(&zmi, &zma, &(*ait->data())[iz->value()][istart], nv);
    autoScale(xmi, xma, ymi, yma, zmi, zma);
    slotEdited();
  }
}

void Array3DDlg::slotButtonClicked(int button)
{
  switch (button) {
    case Apply:
      enableButtonApply(false);
      enableButtonOk(false);
    case Ok: {
      ait->setColX(ix->value());
      ait->setColY(iy->value());
      ait->setColZ(iz->value());
      ait->setColE(ie->value());
      ait->setSmf(eFactor->value());
      ait->setUrl(fileName->text());
      ait->setErrBars(err->isChecked());
      ait->setInternal(internal->isChecked());
      int istart = iStart->value();
      ait->setRowStart(istart);
      ait->setCount(qMin(n->value(), ait->numRows() - istart));
      Array3DItem* aiNew = new Array3DItem(*ait);
      getValues(aiNew);
      m->undoStack()->push(new EditCommand(aiNew, ai, m));
      ai = aiNew;
      if (button == Ok)
        accept();
      break;
    }
    case User1: {
      KUrl url(fileName->text());
      Array3DItem* aisav = ait;
      ait = new Array3DItem(*aisav);
      if (load(url, ait->iDec())) {
        if (Kpl::getWriteUrl(this, url, QString(), "*.dat\n*")) {
          QFile f(url.isLocalFile() ? url.path() : KplItem::tmpFile());
          if (f.open(QIODevice::WriteOnly)) {
            QTextStream ts(&f);
            int maxCol = ait->numCols() - 1;
            ait->setColX(qMin(ix->value(), maxCol));
            ait->setColY(qMin(iy->value(), maxCol));
            ait->setColZ(qMin(iz->value(), maxCol));
            ait->setColE(qMin(ie->value(), maxCol));
            ait->setRowStart(qBound(0, iStart->value(), ait->numRows() - 1));
            ait->setCount(qMin(n->value(), ait->numRows() - ait->rowStart()));
            ait->setSmf(eFactor->value());
            ait->setErrBars(err->isChecked());
            getValues(ait);
            ait->exportTable(ts, KplDoc::separator(), KplDoc::presentation(),
                             KplSettings::precision());
            f.close();
            m->setCurrentDir(url);
            if (!url.isLocalFile())
              m->copyTmp(f.fileName(), url);
          } else
            KMessageBox::error(this, i18n("while trying to open"));
        }
      }
      delete ait;
      ait = aisav;
      break;
    }
    default:
      KDialog::slotButtonClicked(button);
  }
}

void Array3DDlg::slotEdit()
{
  buf.clear();
  QTextStream t(&buf, QIODevice::WriteOnly);
  char format = KplDoc::presentation();
  int prec = KplSettings::precision();
  char sep = KplDoc::separator();
  int ncols = ait->numCols();
  for (int i = 0; i < ait->numRows(); ++i)
    for (int j = 0; j < ncols; ++j) {
      t << QString::number((*ait->data())[j][i], format, prec);
      if (j < (ncols - 1))
        t << sep;
      else
        t << "\n";
    }
  EditDlg dlg(this, &buf, prec + 4);
  connect(&dlg, SIGNAL(applyClicked()), SLOT(slotEditApply()));
  if (dlg.exec())
    if (readData(true))
      setSpinBoxes(ait);
}

void Array3DDlg::slotEditApply()
{
  if (readData(true)) {
    setSpinBoxes(ait);
    slotButtonClicked(Apply);
  }
}

void Array3DDlg::slotFile()
{
  int idec = ait->iDec();
  KUrl url = Kpl::getReadUrl(idec, KplSettings::currentDir() +
                             ait->url().fileName());
  if (load(url, idec)) {
    setSpinBoxes(ait);
    fileName->setText(url.isLocalFile() ? url.path() : url.url());
    m->setCurrentDir(url);
    bReload->setDisabled(fileName->text().isEmpty());
    internal->setChecked(false);
    edit->setEnabled(false);
    slotEdited();
  }
}

void Array3DDlg::slotInternal(bool state)
{
  Array3DItem* src = 0;
  if (state) {
    src = new Array3DItem(*ait);
    ait->setInternal(state);
    ait->setNumCols(4);
    ait->setRowStart(iStart->value());
    ait->setCount(n->value());
    ArrayItem::freeX(ait->data());
    int dim = ait->numCols() * ait->numRows();
    if (dim) {
      double* x1 = new double[dim];
      memset(x1, 0, dim * sizeof(double));
      *ait->data() = new double*[ait->numCols()];
      for (int j = 0; j < ait->numCols(); ++j)
        (*ait->data())[j] = &x1[j * ait->numRows()];
      if (src->numCols() > 4) {
        int siz = ait->numRows() * sizeof(double);
        memcpy((*ait->data())[0], (*src->data())[ix->value()], siz);
        memcpy((*ait->data())[1], (*src->data())[iy->value()], siz);
        memcpy((*ait->data())[2], (*src->data())[iz->value()], siz);
        memcpy((*ait->data())[3], (*src->data())[ie->value()], siz);
        ait->setColX(0);
        ait->setColY(1);
        ait->setColZ(2);
        ait->setColE(3);
      } else {
        memcpy((*ait->data())[0], (*src->data())[0],
               src->numCols() * src->numRows() * sizeof(double));
        ait->setColX(ix->value());
        ait->setColY(iy->value());
        ait->setColZ(iz->value());
        ait->setColE(ie->value());
      }
    }
    setSpinBoxes(ait);
    fileName->clear();
    ait->setIDec(0);
  } else {
    KUrl url;
    if (Kpl::getWriteUrl(this, url, QString(), "*.dat\n*")) {
      QFile f(url.isLocalFile() ? url.path() : KplItem::tmpFile());
      bool success = f.open(QIODevice::WriteOnly);
      if (success) {
        src = new Array3DItem(*ait);
        ait->setInternal(state);
        QTextStream t(&f);
        char format = KplDoc::presentation();
        int prec = KplSettings::precision();
        for (int i = 0; i < src->numRows(); ++i)
          for (int j = 0; j < src->numCols(); ++j)
            t << QString::number((*src->data())[j][i], format, prec)
              << (j < (src->numCols() - 1) ? " " : "\n");
        f.close();
        fileName->setText(url.isLocalFile() ? url.path() : url.url());
        m->setCurrentDir(url);
        if (!url.isLocalFile())
          m->copyTmp(f.fileName(), url);
      } else {
        KMessageBox::error(this, i18n("while trying to open file"));
        internal->setChecked(true);
        return;
      }
    } else {
      internal->setChecked(true);
      return;
    }
  }
  delete src;
  edit->setEnabled(state);
  bReload->setDisabled(fileName->text().isEmpty());
  slotEdited();
}

void Array3DDlg::slotNChanged(int na)
{
  int imin = ai->numRows() - na;
  if (iStart->value() > imin)
    iStart->setValue(imin);
  slotEdited();
}

void Array3DDlg::slotReload()
{
  KUrl url(fileName->text());
  load(url, ait->iDec());
  setSpinBoxes(ait);
  slotEdited();
}

void Array3DDlg::slotStartChanged(int ia)
{
  int nmax = ai->numRows() - ia;
  if (n->value() > nmax)
    n->setValue(nmax);
  slotEdited();
}
